# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

# pylint: disable=line-too-long

import json
import os
import time

from azure.cli.testsdk import (ResourceGroupPreparer, ScenarioTest, KeyVaultPreparer, live_only)
from azure.cli.command_modules.appconfig._constants import KeyVaultConstants
from azure.cli.core.azclierror import RequiredArgumentMissingError, InvalidArgumentValueError, MutuallyExclusiveArgumentError
from azure.cli.testsdk.scenario_tests import AllowLargeResponse
from azure.cli.command_modules.appconfig.tests.latest._test_utils import create_config_store, CredentialResponseSanitizer, get_resource_name_prefix

TEST_DIR = os.path.abspath(os.path.join(os.path.abspath(__file__), '..'))

class AppConfigKVScenarioTest(ScenarioTest):

    def __init__(self, *args, **kwargs):
        kwargs["recording_processors"] = kwargs.get("recording_processors", []) + [CredentialResponseSanitizer()]
        super().__init__(*args, **kwargs)

    @AllowLargeResponse()
    @ResourceGroupPreparer(parameter_name_for_location='location')
    def test_azconfig_kv(self, resource_group, location):
        config_store_prefix = get_resource_name_prefix('KVTest')
        config_store_name = self.create_random_name(prefix=config_store_prefix, length=24)

        location = 'eastus'
        sku = 'standard'
        self.kwargs.update({
            'config_store_name': config_store_name,
            'rg_loc': location,
            'rg': resource_group,
            'sku': sku
        })
        create_config_store(self, self.kwargs)

        entry_key = "Color"
        entry_label = 'v1.0.0'

        self.kwargs.update({
            'key': entry_key,
            'label': entry_label
        })

        # add a new key-value entry
        self.cmd('appconfig kv set -n {config_store_name} --key {key} --label {label} -y',
                 checks=[self.check('contentType', ""),
                         self.check('key', entry_key),
                         self.check('value', ""),
                         self.check('label', entry_label)])

        # edit a key-value entry
        updated_entry_value = "Green"
        entry_content_type = "text"

        self.kwargs.update({
            'value': updated_entry_value,
            'content_type': entry_content_type
        })

        self.cmd('appconfig kv set -n {config_store_name} --key {key} --value {value} --content-type {content_type} --label {label} -y',
                 checks=[self.check('contentType', entry_content_type),
                         self.check('key', entry_key),
                         self.check('value', updated_entry_value),
                         self.check('label', entry_label)])

        # add a new label
        updated_label = 'newlabel'
        self.kwargs.update({
            'label': updated_label
        })

        self.cmd('appconfig kv set -n {config_store_name} --key {key} --value {value} --content-type {content_type} --label {label} -y',
                 checks=[self.check('contentType', entry_content_type),
                         self.check('key', entry_key),
                         self.check('value', updated_entry_value),
                         self.check('label', updated_label)])

        # show a key-value
        self.cmd('appconfig kv show -n {config_store_name} --key {key} --label {label}',
                 checks=[self.check('contentType', entry_content_type),
                         self.check('value', updated_entry_value),
                         self.check('label', updated_label)])

        list_keys = self.cmd(
            'appconfig kv list -n {config_store_name}').get_output_in_json()
        assert len(list_keys) == 2

        revisions = self.cmd(
            'appconfig revision list -n {config_store_name} --key {key} --label *').get_output_in_json()
        assert len(revisions) == 3

        # Confirm that delete action errors out for empty or whitespace key
        with self.assertRaisesRegex(RequiredArgumentMissingError, "Key cannot be empty."):
            self.cmd('appconfig kv delete -n {config_store_name} --key " " -y')

        # IN CLI, since we support delete by key/label filters, return is a list of deleted items
        deleted = self.cmd('appconfig kv delete -n {config_store_name} --key {key} --label {label} -y',
                           checks=[self.check('[0].key', entry_key),
                                   self.check('[0].contentType', entry_content_type),
                                   self.check('[0].value', updated_entry_value),
                                   self.check('[0].label', updated_label)]).get_output_in_json()

        deleted_time = deleted[0]['lastModified']

        # sleep a little over 1 second
        time.sleep(1.1)

        # set key-value entry with connection string, but to the original value
        # take a note of the deleted_time
        entry_value = "Red"

        self.kwargs.update({
            'value': entry_value,
            'timestamp': deleted_time
        })

        credential_list = self.cmd(
            'appconfig credential list -n {config_store_name} -g {rg}').get_output_in_json()
        self.kwargs.update({
            'connection_string': credential_list[0]['connectionString']
        })
        self.cmd('appconfig kv set --connection-string {connection_string} --key {key} --value {value} --content-type {content_type} --label {label} -y',
                 checks=[self.check('contentType', entry_content_type),
                         self.check('key', entry_key),
                         self.check('value', entry_value),
                         self.check('label', updated_label)])

        # Now restore to last modified and ensure that we find updated_entry_value
        self.cmd('appconfig kv restore -n {config_store_name} --key {key} --label {label} --datetime {timestamp} -y')
        self.cmd('appconfig kv list -n {config_store_name} --key {key} --label {label}',
                 checks=[self.check('[0].contentType', entry_content_type),
                         self.check('[0].key', entry_key),
                         self.check('[0].value', updated_entry_value),
                         self.check('[0].label', updated_label)])

        # KeyVault reference tests
        keyvault_key = "HostSecrets"
        keyvault_id = "https://fake.vault.azure.net/secrets/fakesecret"
        keyvault_value = f"{{{json.dumps({'uri': keyvault_id})}}}"

        self.kwargs.update({
            'key': keyvault_key,
            'secret_identifier': keyvault_id
        })

        # Add new KeyVault ref
        self.cmd('appconfig kv set-keyvault --connection-string {connection_string} --key {key} --secret-identifier {secret_identifier} -y',
                 checks=[self.check('contentType', KeyVaultConstants.KEYVAULT_CONTENT_TYPE),
                         self.check('key', keyvault_key),
                         self.check('value', keyvault_value)])

        # Update existing key to KeyVault ref
        self.kwargs.update({
            'key': entry_key,
            'label': updated_label
        })

        self.cmd('appconfig kv set-keyvault --connection-string {connection_string} --key {key} --label {label} --secret-identifier {secret_identifier} -y',
                 checks=[self.check('contentType', KeyVaultConstants.KEYVAULT_CONTENT_TYPE),
                         self.check('key', entry_key),
                         self.check('value', keyvault_value),
                         self.check('label', updated_label)])

        # Delete KeyVault ref
        self.cmd('appconfig kv delete --connection-string {connection_string} --key {key} --label {label} -y',
                 checks=[self.check('[0].key', entry_key),
                         self.check('[0].contentType', KeyVaultConstants.KEYVAULT_CONTENT_TYPE),
                         self.check('[0].value', keyvault_value),
                         self.check('[0].label', updated_label)])

        # add a key-value with null label
        kv_with_null_label = 'KvWithNullLabel'
        self.kwargs.update({
            'key': kv_with_null_label
        })

        self.cmd('appconfig kv set --connection-string {connection_string} --key {key} -y',
                 checks=[self.check('key', kv_with_null_label),
                         self.check('label', None)])

        # List key-values with null label
        null_label_pattern = "\\0"
        self.kwargs.update({
            'null_label': null_label_pattern
        })
        list_keys = self.cmd(
            'appconfig kv list --connection-string {connection_string} --label "{null_label}"').get_output_in_json()
        assert len(list_keys) == 2

        # List key-values with multiple labels
        multi_labels = entry_label + ',' + null_label_pattern
        self.kwargs.update({
            'multi_labels': multi_labels
        })
        list_keys = self.cmd(
            'appconfig kv list --connection-string {connection_string} --label "{multi_labels}"').get_output_in_json()
        assert len(list_keys) == 3
    
        # # Filter by tags test

        # Set key-value with 5 tags
        key_with_tags = "KeyWithTags"
        label_with_tags = "labelWithTags"
        tags = {
            "tag1": "value1",
            "tag2": "value2",
            "tag3": "value3",
            "tag4": "value4",
            "tag5": "value5"
        }
        tags_str = ' '.join([f"{k}={v}" for k, v in tags.items()])

        self.kwargs.update({
            'key': key_with_tags,
            'label': label_with_tags,
            'tags': tags_str
        })

        self.cmd('appconfig kv set --connection-string {connection_string} --key {key} --label {label} --tags {tags} -y',
             checks=[self.check('key', key_with_tags),
             self.check('label', label_with_tags),
            self.check('tags', tags)])

        # List key-values with 5 tags
        list_keys = self.cmd('appconfig kv list --connection-string {connection_string} --key {key} --tags {tags}').get_output_in_json()
        assert(list_keys[0]['key'] == key_with_tags)
        assert(list_keys[0]['label'] == label_with_tags)
        assert(list_keys[0]['tags'] == tags)
        assert len(list_keys) == 1

        # Set key-value with tag1 only
        tag1_dict = {"tag1": "value1"}
        tag1 = ' '.join([f"{k}={v}" for k, v in tag1_dict.items()])
        label_with_tag1 = "labelWithTag1"
        self.kwargs.update({
            'label': label_with_tag1,
            'tag1': tag1
        })

        self.cmd('appconfig kv set --connection-string {connection_string} --key {key} --label {label} --tags {tag1} -y',
             checks=[self.check('key', key_with_tags),
                 self.check('label', label_with_tag1),
                 self.check('tags', tag1_dict)])

        # List key-values with tag1
        list_keys = self.cmd('appconfig kv list --connection-string {connection_string} --key {key} --tags {tag1}').get_output_in_json()
        assert len(list_keys) == 2

        # Set key-value with empty tag value
        empty_tag_value_dict = {"tag1": ""}
        tag_with_empty_value = ' '.join([f"{k}={v}" for k, v in empty_tag_value_dict.items()])
        label_with_empty_tag_value = "labelWithEmptyTagValue"
        self.kwargs.update({
            'label': label_with_empty_tag_value,
            'tag_with_empty_value': tag_with_empty_value
        })

        self.cmd('appconfig kv set --connection-string {connection_string} --key {key} --label {label} --tags {tag_with_empty_value} -y',
             checks=[self.check('key', key_with_tags),
                 self.check('label', label_with_empty_tag_value),
                 self.check('tags', empty_tag_value_dict)])

        # List key-values with tag with empty value
        list_keys = self.cmd('appconfig kv list --connection-string {connection_string} --key {key} --tags {tag_with_empty_value}').get_output_in_json()
        assert len(list_keys) == 1

        # Get all key-values with all tags
        list_keys = self.cmd('appconfig kv list --connection-string {connection_string} --key {key}').get_output_in_json()
        assert len(list_keys) == 3

        # Delete key-values with tags tag1=value1
        self.kwargs.update({
            'label': '*'
        })

        deleted_kvs = self.cmd('appconfig kv delete --connection-string {connection_string} --key {key} --label {label} --tags {tag1} -y',
             checks=[self.check('[0].key', key_with_tags),
                    self.check('[0].label', label_with_tag1),
                    self.check('[0].tags', tag1_dict)]).get_output_in_json()
        assert(len(deleted_kvs) == 2)

        # Error if more than 5 tags are provided
        too_many_tags = {
            "tag1": "value1",
            "tag2": "value2",
            "tag3": "value3",
            "tag4": "value4",
            "tag5": "value5",
            "tag6": "value6"
        }
        too_many_tags_str = ' '.join([f"{k}={v}" for k, v in too_many_tags.items()])

        self.kwargs.update({
            'too_many_tags': too_many_tags_str
        })

        with self.assertRaisesRegex(InvalidArgumentValueError, "Too many tag filters provided. Maximum allowed is 5."):
            self.cmd('appconfig kv list --connection-string {connection_string} --tags {too_many_tags}')

        # Dry run and yes argument should not be used together
        with self.assertRaisesRegex(MutuallyExclusiveArgumentError, "The '--dry-run' and '--yes' options cannot be specified together."):
            self.cmd('appconfig kv restore --datetime "2019-05-01T11:24:12Z" --connection-string {connection_string} --key {key} --label {label} --dry-run -y')


    @AllowLargeResponse()
    @ResourceGroupPreparer()
    @KeyVaultPreparer(additional_params="--enable-rbac-authorization false")
    @live_only()
    def test_resolve_keyvault(self, key_vault, resource_group):
        config_store_prefix = get_resource_name_prefix('KVTest')
        config_store_name = self.create_random_name(prefix=config_store_prefix, length=24)

        location = 'eastus'
        sku = 'standard'
        self.kwargs.update({
            'config_store_name': config_store_name,
            'rg_loc': location,
            'rg': resource_group,
            'sku': sku
        })
        create_config_store(self, self.kwargs)

        # Export secret test
        secret_name = 'testSecret'
        secret_value = 'testValue'
        self.kwargs.update({
            'secret_name': secret_name,
            'secret_value': secret_value,
            'keyvault_name': key_vault
        })

        secret = self.cmd('az keyvault secret set --vault-name {keyvault_name} -n {secret_name} --value {secret_value}').get_output_in_json()
        self.kwargs.update({
            'secret_identifier': secret["id"]
        })

        self.cmd('appconfig kv set-keyvault -n {config_store_name} --key {secret_name} --secret-identifier {secret_identifier} -y')

        self.cmd('appconfig kv list -n {config_store_name} --resolve-keyvault',
                 checks=[self.check('[0].key', secret_name),
                         self.check('[0].value', secret_value)])

        exported_file_path = os.path.join(TEST_DIR, 'export_keyvault.json')

        self.kwargs.update({
            'import_source': 'file',
            'exported_file_path': exported_file_path,
            'imported_format': 'json',
        })

        self.cmd('appconfig kv export -n {config_store_name} -d file --path "{exported_file_path}" --format json --resolve-keyvault -y')
        with open(exported_file_path) as json_file:
            exported_kvs = json.load(json_file)

        assert len(exported_kvs) == 1
        assert exported_kvs[secret_name] == secret_value
        os.remove(exported_file_path)

    @AllowLargeResponse()
    @ResourceGroupPreparer(parameter_name_for_location='location')
    def test_azconfig_kv_revision_list(self, resource_group, location):
        config_store_prefix = get_resource_name_prefix('KVRevisionTest')
        config_store_name = self.create_random_name(prefix=config_store_prefix, length=24)

        location = 'eastus'
        sku = 'standard'
        self.kwargs.update({
            'config_store_name': config_store_name,
            'rg_loc': location,
            'rg': resource_group,
            'sku': sku
        })
        create_config_store(self, self.kwargs)

        entry_key = "Color"
        entry_label = 'v1.0.0'

        self.kwargs.update({
            'key': entry_key,
            'label': entry_label
        })

        # add a new key-value entry
        self.cmd('appconfig kv set -n {config_store_name} --key {key} --label {label} -y',
                 checks=[self.check('contentType', ""),
                         self.check('key', entry_key),
                         self.check('value', ""),
                         self.check('label', entry_label)])

        # edit a key-value entry
        updated_entry_value = "Green"
        entry_content_type = "text"

        self.kwargs.update({
            'value': updated_entry_value,
            'content_type': entry_content_type
        })

        self.cmd(
            'appconfig kv set -n {config_store_name} --key {key} --value {value} --content-type {content_type} --label {label} -y',
            checks=[self.check('contentType', entry_content_type),
                    self.check('key', entry_key),
                    self.check('value', updated_entry_value),
                    self.check('label', entry_label)])

        # add a new label
        updated_label = 'newlabel'
        self.kwargs.update({
            'label': updated_label
        })

        self.cmd(
            'appconfig kv set -n {config_store_name} --key {key} --value {value} --content-type {content_type} --label {label} -y',
            checks=[self.check('contentType', entry_content_type),
                    self.check('key', entry_key),
                    self.check('value', updated_entry_value),
                    self.check('label', updated_label)])

        revisions = self.cmd('appconfig revision list -n {config_store_name} --key {key} --label * --top 2 --fields content_type etag label last_modified value').get_output_in_json()
        assert len(revisions) == 2

        assert revisions[0]['content_type'] == 'text'
        assert revisions[1]['content_type'] == 'text'
        assert revisions[0]['label'] == 'newlabel'
        assert revisions[1]['label'] == 'v1.0.0'
        assert revisions[0]['value'] == 'Green'
        assert revisions[1]['value'] == 'Green'
        assert revisions[0]['last_modified'] is not None
        assert revisions[1]['last_modified'] is not None
        assert revisions[1]['etag'] is not None
        assert revisions[0]['etag'] is not None
        assert 'key' not in revisions[0]
        assert 'key' not in revisions[1]
        assert 'locked' not in revisions[0]
        assert 'locked' not in revisions[1]
        assert 'tags' not in revisions[0]
        assert 'tags' not in revisions[1]

        # Filter by tags test
        key_with_tags = "KeyWithTags"
        label_with_tags = "labelWithTags"
        tags = {
            "tag1": "value1",
            "tag2": "value2",
            "tag3": "value3",
            "tag4": "value4",
            "tag5": "value5"
        }
        tags_str = ' '.join([f"{k}={v}" for k, v in tags.items()])

        self.kwargs.update({
            'key': key_with_tags,
            'label': label_with_tags,
            'tags': tags_str
        })

        self.cmd('appconfig kv set -n {config_store_name} --key {key} --label {label} --tags {tags} -y',
             checks=[self.check('key', key_with_tags),
                 self.check('label', label_with_tags),
                 self.check('tags', tags)])

        # List revisions with 5 tags
        list_keys = self.cmd('appconfig revision list -n {config_store_name} --key {key} --tags {tags}').get_output_in_json()
        assert len(list_keys) == 1

        # Set key-value with tag1 only
        tag1_dict = {"tag1": "value1"}
        tag1 = ' '.join([f"{k}={v}" for k, v in tag1_dict.items()])
        label_with_tag1 = "labelWithTag1"
        self.kwargs.update({
            'label': label_with_tag1,
            'tag1': tag1
        })

        self.cmd('appconfig kv set -n {config_store_name} --key {key} --label {label} --tags {tag1} -y',
             checks=[self.check('key', key_with_tags),
                 self.check('label', label_with_tag1),
                    self.check('tags', tag1_dict)])

        # List revisions with tag1 = value1
        list_keys = self.cmd('appconfig revision list -n {config_store_name} --key {key} --tags {tag1}').get_output_in_json()
        assert len(list_keys) == 2

        # Set key-value with empty tag value
        empty_tag_value_dict = {"tag1": ""}
        tag_with_empty_value = ' '.join([f"{k}={v}" for k, v in empty_tag_value_dict.items()])
        self.kwargs.update({
            'tag_with_empty_value': tag_with_empty_value
        })

        self.cmd('appconfig kv set -n {config_store_name} --key {key} --label {label} --tags {tag_with_empty_value} -y',
             checks=[self.check('key', key_with_tags),
                    self.check('tags', empty_tag_value_dict)])

        # List revisions with tag with empty value
        list_keys = self.cmd('appconfig revision list -n {config_store_name} --key {key} --tags {tag_with_empty_value}').get_output_in_json()
        assert len(list_keys) == 1

        # Get all key-value revisions for key with all tags
        revisions = self.cmd('appconfig revision list -n {config_store_name} --key {key} --label *').get_output_in_json()
        assert len(revisions) == 3



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

import unittest

from unittest import mock
import os
import tempfile

from azure.cli.core import AzCommandsLoader
from azure.cli.core.commands import AzCliCommand
from azure.cli.core.commands.validators import IterateAction

from azure.cli.core.mock import DummyCli

from knack.util import CLIError


class TestApplication(unittest.TestCase):
    def test_client_request_id_is_not_assigned_when_application_is_created(self):
        cli = DummyCli()
        self.assertNotIn('x-ms-client-request-id', cli.data['headers'])

    def test_client_request_id_is_refreshed_correctly(self):
        cli = DummyCli()
        cli.refresh_request_id()
        self.assertIn('x-ms-client-request-id', cli.data['headers'])

        old_id = cli.data['headers']['x-ms-client-request-id']

        cli.refresh_request_id()
        self.assertIn('x-ms-client-request-id', cli.data['headers'])
        self.assertNotEqual(old_id, cli.data['headers']['x-ms-client-request-id'])

    def test_client_request_id_is_refreshed_after_execution(self):
        def _handler(args):
            return True

        class TestCommandsLoader(AzCommandsLoader):

            def load_command_table(self, args):
                super().load_command_table(args)
                self.command_table = {'test': AzCliCommand(self, 'test', _handler)}
                return self.command_table

        cli = DummyCli(commands_loader_cls=TestCommandsLoader)

        cli.invoke(['test'])
        self.assertIn('x-ms-client-request-id', cli.data['headers'])
        old_id = cli.data['headers']['x-ms-client-request-id']

        cli.invoke(['test'])
        self.assertIn('x-ms-client-request-id', cli.data['headers'])
        self.assertNotEqual(old_id, cli.data['headers']['x-ms-client-request-id'])

    def test_application_register_and_call_handlers(self):
        handler_called = [False]

        def handler(*args, **kwargs):
            kwargs['args'][0] = True

        def other_handler(*args, **kwargs):
            self.assertEqual(kwargs['args'], 'secret sauce')

        cli = DummyCli()

        cli.raise_event('was_handler_called', args=handler_called)
        self.assertFalse(handler_called[0], "Raising event with no handlers registered somehow failed...")

        cli.register_event('was_handler_called', handler)
        self.assertFalse(handler_called[0])

        # Registered handler won't get called if event with different name
        # is raised...
        cli.raise_event('other_handler_called', args=handler_called)
        self.assertFalse(handler_called[0], 'Wrong handler called!')

        cli.raise_event('was_handler_called', args=handler_called)
        self.assertTrue(handler_called[0], "Handler didn't get called")

        cli.raise_event('other_handler_called', args='secret sauce')

    def test_expand_file_prefixed_files(self):
        f = tempfile.NamedTemporaryFile(delete=False)
        f.close()

        f_with_bom = tempfile.NamedTemporaryFile(delete=False)
        f_with_bom.close()

        with open(f.name, 'w+') as stream:
            stream.write('foo')

        from codecs import open as codecs_open
        with codecs_open(f_with_bom.name, encoding='utf-8-sig', mode='w+') as stream:
            stream.write('foo')

        cases = [
            [['bar=baz'], ['bar=baz']],
            [['bar', 'baz'], ['bar', 'baz']],
            [['bar=@{}'.format(f.name)], ['bar=foo']],
            [['bar=@{}'.format(f_with_bom.name)], ['bar=foo']],
            [['bar', '@{}'.format(f.name)], ['bar', 'foo']],
            [['bar', f.name], ['bar', f.name]],
            [['bar=name@company.com'], ['bar=name@company.com']],
            [['bar', 'name@company.com'], ['bar', 'name@company.com']],
            [['bar=mymongo=@connectionstring'], ['bar=mymongo=@connectionstring']],
            [['bar=@noneexisting'], ['bar=@noneexisting']]
        ]

        for test_case in cases:
            try:
                from azure.cli.core.commands import _expand_file_prefixed_files
                args = _expand_file_prefixed_files(test_case[0])  # pylint: disable=protected-access
                self.assertEqual(args, test_case[1], 'Failed for: {}'.format(test_case[0]))
            except CLIError as ex:
                self.fail('Unexpected error for {} ({}): {}'.format(test_case[0], args, ex))

        os.remove(f.name)


if __name__ == '__main__':
    unittest.main()



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

from azure.cli.command_modules.aro._actions import AROPlatformWorkloadIdentityAddAction
from azure.cli.command_modules.aro._validators import (
    validate_cidr,
    validate_client_id,
    validate_client_secret,
    validate_cluster_resource_group,
    validate_disk_encryption_set,
    validate_domain,
    validate_pull_secret,
    validate_subnet,
    validate_visibility,
    validate_vnet,
    validate_vnet_resource_group_name,
    validate_worker_count,
    validate_worker_vm_disk_size_gb,
    validate_refresh_cluster_credentials,
    validate_version_format,
    validate_outbound_type,
    validate_load_balancer_managed_outbound_ip_count,
    validate_enable_managed_identity,
    validate_platform_workload_identities,
    validate_cluster_identity,
    validate_cluster_identity_update,
    validate_delete_identities,
    validate_upgradeable_to_format,
)
from azure.cli.core.commands.parameters import (
    name_type,
    get_enum_type,
    get_three_state_flag,
    resource_group_name_type,
    tags_type
)
from azure.cli.core.commands.validators import get_default_location_from_resource_group


def load_arguments(self, _):
    with self.argument_context('aro') as c:
        c.argument('location',
                   validator=get_default_location_from_resource_group)
        c.argument('resource_name',
                   name_type,
                   help='Name of cluster.')
        c.argument('tags',
                   tags_type,
                   help='Tags to be assigned to the openShiftCluster resource. ' + tags_type.settings.get('help', ''))

        c.argument('pull_secret',
                   help='Pull secret of cluster.',
                   validator=validate_pull_secret)
        c.argument('domain',
                   help='Domain of cluster.',
                   validator=validate_domain)
        c.argument('cluster_resource_group',
                   help='Resource group of cluster.',
                   validator=validate_cluster_resource_group)
        c.argument('fips_validated_modules', arg_type=get_three_state_flag(),
                   options_list=['--fips-validated-modules', '--fips'],
                   help='Use FIPS validated cryptography modules. [Default: false]')

        c.argument('client_id',
                   help='Client ID of cluster service principal.',
                   validator=validate_client_id(isCreate=True))
        c.argument('client_secret',
                   help='Client secret of cluster service principal.',
                   validator=validate_client_secret(isCreate=True))

        c.argument('version',
                   options_list=['--version', c.deprecate(target='--install-version', redirect='--version', hide=True)],
                   help='OpenShift version to use for cluster creation.',
                   validator=validate_version_format)

        c.argument('pod_cidr',
                   help='CIDR of pod network. Must be a minimum of /18 or larger. [Default: 10.128.0.0/14]',
                   validator=validate_cidr('pod_cidr'))
        c.argument('service_cidr',
                   help='CIDR of service network. Must be a minimum of /18 or larger. [Default: 172.30.0.0/16]',
                   validator=validate_cidr('service_cidr'))
        c.argument('outbound_type',
                   help='Outbound type of cluster. Must be "Loadbalancer" or "UserDefinedRouting". \
                   [Default: Loadbalancer]',
                   validator=validate_outbound_type)
        c.argument('enable_preconfigured_nsg', arg_type=get_three_state_flag(),
                   help='Use Preconfigured NSGs. Allowed values: false, true. [Default: false]')
        c.argument('disk_encryption_set',
                   help='ResourceID of the DiskEncryptionSet to be used for master and worker VMs.',
                   validator=validate_disk_encryption_set)
        c.argument('master_encryption_at_host', arg_type=get_three_state_flag(),
                   options_list=['--master-encryption-at-host', '--master-enc-host'],
                   help='Encryption at host flag for master VMs. [Default: false]')
        c.argument('master_vm_size',
                   help='Size of master VMs. [Default: Standard_D8s_v5]')

        c.argument('worker_encryption_at_host', arg_type=get_three_state_flag(),
                   options_list=['--worker-encryption-at-host', '--worker-enc-host'],
                   help='Encryption at host flag for worker VMs. [Default: false]')
        c.argument('worker_vm_size',
                   help='Size of worker VMs.')
        c.argument('worker_vm_disk_size_gb',
                   type=int,
                   help='Disk size in GB of worker VMs. [Default: 128]',
                   validator=validate_worker_vm_disk_size_gb)
        c.argument('worker_count',
                   type=int,
                   help='Count of worker VMs. [Default: 3]',
                   validator=validate_worker_count)

        c.argument('apiserver_visibility', arg_type=get_enum_type(['Private', 'Public']),
                   help='API server visibility. [Default: Public]',
                   validator=validate_visibility('apiserver_visibility'))

        c.argument('ingress_visibility', arg_type=get_enum_type(['Private', 'Public']),
                   help='Ingress visibility. [Default: Public]',
                   validator=validate_visibility('ingress_visibility'))

        c.argument('vnet_resource_group_name',
                   resource_group_name_type,
                   options_list=['--vnet-resource-group'],
                   help='Name of vnet resource group.',
                   validator=validate_vnet_resource_group_name)
        c.argument('vnet',
                   help='Name or ID of vnet.  If name is supplied, `--vnet-resource-group` must be supplied.',
                   validator=validate_vnet)
        c.argument('master_subnet',
                   help='Name or ID of master vnet subnet.  If name is supplied, `--vnet` must be supplied.',
                   validator=validate_subnet('master_subnet'))
        c.argument('worker_subnet',
                   help='Name or ID of worker vnet subnet.  If name is supplied, `--vnet` must be supplied.',
                   validator=validate_subnet('worker_subnet'))
        c.argument('load_balancer_managed_outbound_ip_count',
                   type=int,
                   help='The desired number of IPv4 outbound IPs created and managed by Azure for the cluster public load balancer.',  # pylint: disable=line-too-long
                   validator=validate_load_balancer_managed_outbound_ip_count,
                   options_list=['--load-balancer-managed-outbound-ip-count', '--lb-ip-count'])

        c.argument('enable_managed_identity', arg_group='Identity', arg_type=get_three_state_flag(),
                   help='Enable managed identity for this cluster.',
                   options_list=['--enable-managed-identity', '--enable-mi'],
                   validator=validate_enable_managed_identity)
        c.argument('platform_workload_identities', arg_group='Identity',
                   help='Assign a platform workload identity used within the cluster. Requires two values: \
                           an operator name and either the name or resource ID of the Azure identity to use for it.',
                   options_list=['--assign-platform-workload-identity', '--assign-platform-wi'],
                   validator=validate_platform_workload_identities(isCreate=True),
                   action=AROPlatformWorkloadIdentityAddAction, nargs='+')
        c.argument('mi_user_assigned', arg_group='Identity',
                   help='Set the user managed identity on the cluster. Value must be an identity name or resource ID.',
                   options_list=['--mi-user-assigned', '--assign-cluster-identity'],
                   validator=validate_cluster_identity)

    with self.argument_context('aro update') as c:
        c.argument('client_id',
                   help='Client ID of cluster service principal.',
                   validator=validate_client_id(isCreate=False))
        c.argument('client_secret',
                   help='Client secret of cluster service principal.',
                   validator=validate_client_secret(isCreate=False))
        c.argument('refresh_cluster_credentials',
                   arg_type=get_three_state_flag(),
                   help='Refresh cluster application credentials.',
                   options_list=['--refresh-credentials'],
                   validator=validate_refresh_cluster_credentials)
        c.argument('mi_user_assigned', arg_group='Identity',
                   help='Set the user managed identity on the cluster. Value must be an identity name or resource ID.',
                   options_list=['--mi-user-assigned', '--assign-cluster-identity'],
                   validator=validate_cluster_identity_update)
        c.argument('platform_workload_identities', arg_group='Identity',
                   help='Assign a platform workload identity used within the cluster. Requires two values: \
                           an operator name and either the name or resource ID of the Azure identity to use for it.',
                   options_list=['--assign-platform-workload-identity', '--assign-platform-wi'],
                   validator=validate_platform_workload_identities(isCreate=False),
                   action=AROPlatformWorkloadIdentityAddAction, nargs='+')
        c.argument('upgradeable_to', arg_group='Identity', options_list=['--upgradeable-to'],
                   help='OpenShift version to upgrade to.',
                   validator=validate_upgradeable_to_format)

    with self.argument_context('aro get-admin-kubeconfig') as c:
        c.argument('file',
                   help='Path to the file where kubeconfig should be saved. Default: kubeconfig in local directory',
                   options_list=['--file', '-f'])

    with self.argument_context('aro delete') as c:
        c.argument('delete_identities',
                   arg_group='Identity',
                   arg_type=get_three_state_flag(),
                   validator=validate_delete_identities,
                   help='Delete the cluster\'s associated managed identities together with the cluster.')



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------
import os
import tempfile
import unittest
import shutil
import zipfile

from unittest import mock
from azure.cli.core.mock import DummyCli

from azure.cli.core.extension import (get_extensions, build_extension_path, extension_exists,
                                      get_extension, get_extension_names, get_extension_modname, ext_compat_with_cli,
                                      ExtensionNotInstalledException, WheelExtension,
                                      EXTENSIONS_MOD_PREFIX, EXT_METADATA_MINCLICOREVERSION, EXT_METADATA_MAXCLICOREVERSION)

from azure.cli.core.extension.operations import list_available_extensions, add_extension, show_extension, remove_extension, list_extensions, update_extension

# The test extension name
EXT_NAME = 'myfirstcliextension'
EXT_VERSION = '0.0.3+dev'
SECOND_EXT_NAME = 'my_second_cli_extension'


def _get_test_data_file(filename):
    return os.path.join(os.path.dirname(os.path.realpath(__file__)), 'data', filename)


def _install_test_extension1(system=None):  # pylint: disable=no-self-use
    # We extract the extension into place as we aren't testing install here
    zip_file = _get_test_data_file('{}.zip'.format(EXT_NAME))
    zip_ref = zipfile.ZipFile(zip_file, 'r')
    zip_ref.extractall(build_extension_path(EXT_NAME, system=system))
    zip_ref.close()


def _install_test_extension2(system=None):  # pylint: disable=no-self-use
    # We extract the extension into place as we aren't testing install here
    zip_file = _get_test_data_file('myfirstcliextension_az_extmetadata.zip')
    zip_ref = zipfile.ZipFile(zip_file, 'r')
    zip_ref.extractall(build_extension_path(EXT_NAME, system=system))
    zip_ref.close()


def _install_test_extension3(system=None):  # pylint: disable=no-self-use
    # We extract the extension into place as we aren't testing install here
    zip_file = _get_test_data_file('{}.zip'.format(SECOND_EXT_NAME))
    zip_ref = zipfile.ZipFile(zip_file, 'r')
    zip_ref.extractall(build_extension_path(SECOND_EXT_NAME, system=system))
    zip_ref.close()


def mock_ext(filename, version=None, download_url=None, digest=None, project_url=None, name=None, min_cli_version=None, max_cli_version=None):
    d = {
        'filename': filename,
        'metadata': {
            'name': name,
            'version': version,
            'extensions': {
                'python.details': {
                    'project_urls': {
                        'Home': project_url or 'https://github.com/azure/some-extension'
                    }
                }
            },
            EXT_METADATA_MINCLICOREVERSION: min_cli_version,
            EXT_METADATA_MAXCLICOREVERSION: max_cli_version,
        },
        'downloadUrl': download_url or 'http://contoso.com/{}'.format(filename),
        'sha256Digest': digest
    }
    return d


class TestExtensionsBase(unittest.TestCase):

    def setUp(self):
        self.ext_dir = tempfile.mkdtemp()
        self.ext_sys_dir = tempfile.mkdtemp()
        self.patchers = [mock.patch('azure.cli.core.extension.EXTENSIONS_DIR', self.ext_dir),
                         mock.patch('azure.cli.core.extension.EXTENSIONS_SYS_DIR', self.ext_sys_dir)]
        for patcher in self.patchers:
            patcher.start()
        cmd = mock.MagicMock()
        cmd.cli_ctx = DummyCli()
        self.cmd = cmd

    def tearDown(self):
        for patcher in self.patchers:
            patcher.stop()
        shutil.rmtree(self.ext_dir, ignore_errors=True)
        shutil.rmtree(self.ext_sys_dir, ignore_errors=True)


class TestExtensions(TestExtensionsBase):

    def test_no_extensions_dir(self):
        """ Extensions directory doesn't exist """
        shutil.rmtree(self.ext_dir)
        actual = get_extensions(ext_type=WheelExtension)
        self.assertEqual(len(actual), 0)

    def test_no_extensions_in_dir(self):
        """ Directory exists but there are no extensions """
        actual = get_extensions(ext_type=WheelExtension)
        self.assertEqual(len(actual), 0)

    def test_other_files_in_extensions_dir(self):
        tempfile.mkstemp(dir=self.ext_dir)
        actual = get_extensions(ext_type=WheelExtension)
        self.assertEqual(len(actual), 0)

    def test_extension_list(self):
        _install_test_extension1()
        actual = get_extensions(ext_type=WheelExtension)
        self.assertEqual(len(actual), 1)

    def test_extension_exists(self):
        _install_test_extension1()
        actual = extension_exists(EXT_NAME, ext_type=WheelExtension)
        self.assertTrue(actual)

    def test_extension_not_exists(self):
        actual = extension_exists(EXT_NAME, ext_type=WheelExtension)
        self.assertFalse(actual)

    def test_extension_not_exists_2(self):
        _install_test_extension1()
        actual = extension_exists('notanextension', ext_type=WheelExtension)
        self.assertFalse(actual)

    def test_get_extension(self):
        _install_test_extension1()
        actual = get_extension(EXT_NAME, ext_type=WheelExtension)
        self.assertEqual(actual.name, EXT_NAME)

    def test_get_extension_not_installed(self):
        with self.assertRaises(ExtensionNotInstalledException):
            get_extension(EXT_NAME, ext_type=WheelExtension)

    def test_get_extension_names(self):
        _install_test_extension1()
        actual = get_extension_names(ext_type=WheelExtension)
        self.assertEqual(len(actual), 1)
        self.assertEqual(actual[0], EXT_NAME)

    def test_get_extension_modname_no_mods_with_prefix(self):
        tmp_dir = tempfile.mkdtemp()
        with self.assertRaises(AssertionError):
            get_extension_modname(ext_dir=tmp_dir)

    def test_get_extension_modname_okay(self):
        tmp_dir = tempfile.mkdtemp()
        expected_modname = EXTENSIONS_MOD_PREFIX + 'helloworldmod'
        os.makedirs(os.path.join(tmp_dir, expected_modname))
        actual_modname = get_extension_modname(ext_dir=tmp_dir)
        self.assertEqual(expected_modname, actual_modname)

    def test_get_extension_modname_too_many_mods_with_prefix(self):
        tmp_dir = tempfile.mkdtemp()
        modname1 = EXTENSIONS_MOD_PREFIX + 'helloworldmod1'
        modname2 = EXTENSIONS_MOD_PREFIX + 'helloworldmod2'
        os.makedirs(os.path.join(tmp_dir, modname1))
        os.makedirs(os.path.join(tmp_dir, modname2))
        with self.assertRaises(AssertionError):
            get_extension_modname(ext_dir=tmp_dir)

    def test_get_extension_modname_file_with_prefix(self):
        # We should only file a module if it's a directory and not a file even if it has the prefix
        tmp_dir = tempfile.mkdtemp()
        filename = EXTENSIONS_MOD_PREFIX + 'helloworld'
        open(os.path.join(tmp_dir, filename), 'a').close()
        with self.assertRaises(AssertionError):
            get_extension_modname(ext_dir=tmp_dir)

    def test_ext_compat_with_cli_no_v_constraint(self):
        # An extension that does not specify any version constraint on the CLI
        expected_cli_version = '0.0.1'
        azext_metadata = None
        with mock.patch('azure.cli.core.__version__', expected_cli_version):
            is_compatible, cli_version, _, _, _ = ext_compat_with_cli(azext_metadata)
            self.assertTrue(is_compatible)
            self.assertEqual(cli_version, expected_cli_version)

    def test_ext_compat_with_cli_single_v_constraint(self):
        # An extension that only works with a specific version of the CLI
        expected_cli_version = '0.0.1'
        expected_min_required = '0.0.1'
        expected_max_required = '0.0.1'
        azext_metadata = {EXT_METADATA_MINCLICOREVERSION: expected_min_required,
                          EXT_METADATA_MAXCLICOREVERSION: expected_max_required}
        with mock.patch('azure.cli.core.__version__', expected_cli_version):
            is_compatible, cli_version, min_required, max_required, _ = ext_compat_with_cli(azext_metadata)
            self.assertTrue(is_compatible)
            self.assertEqual(cli_version, expected_cli_version)
            self.assertEqual(min_required, expected_min_required)
            self.assertEqual(max_required, expected_max_required)

    def test_ext_compat_with_cli_only_min_v_constraint(self):
        expected_cli_version = '0.0.5'
        expected_min_required = '0.0.1'
        azext_metadata = {EXT_METADATA_MINCLICOREVERSION: expected_min_required}
        with mock.patch('azure.cli.core.__version__', expected_cli_version):
            is_compatible, _, _, _, _ = ext_compat_with_cli(azext_metadata)
            self.assertTrue(is_compatible)

    def test_ext_compat_with_cli_failed_bad_min_v_constraint(self):
        expected_cli_version = '0.0.5'
        expected_min_required = '0.0.7'
        azext_metadata = {EXT_METADATA_MINCLICOREVERSION: expected_min_required}
        with mock.patch('azure.cli.core.__version__', expected_cli_version):
            is_compatible, _, _, _, _ = ext_compat_with_cli(azext_metadata)
            self.assertFalse(is_compatible)

    def test_ext_compat_with_cli_failed_bad_min_but_close_v_constraint(self):
        expected_cli_version = '0.0.5'
        expected_min_required = '0.0.5+dev'
        azext_metadata = {EXT_METADATA_MINCLICOREVERSION: expected_min_required}
        with mock.patch('azure.cli.core.__version__', expected_cli_version):
            is_compatible, _, _, _, _ = ext_compat_with_cli(azext_metadata)
            self.assertFalse(is_compatible)

    def test_ext_compat_with_cli_only_max_v_constraint(self):
        expected_cli_version = '0.0.5'
        expected_max_required = '0.0.10'
        azext_metadata = {EXT_METADATA_MAXCLICOREVERSION: expected_max_required}
        with mock.patch('azure.cli.core.__version__', expected_cli_version):
            is_compatible, _, _, _, _ = ext_compat_with_cli(azext_metadata)
            self.assertTrue(is_compatible)

    def test_ext_compat_with_cli_failed_bad_max_v_constraint(self):
        expected_cli_version = '0.0.5'
        expected_max_required = '0.0.3'
        azext_metadata = {EXT_METADATA_MAXCLICOREVERSION: expected_max_required}
        with mock.patch('azure.cli.core.__version__', expected_cli_version):
            is_compatible, _, _, _, _ = ext_compat_with_cli(azext_metadata)
            self.assertFalse(is_compatible)

    def test_ext_compat_with_cli_failed_bad_max_but_close_v_constraint(self):
        expected_cli_version = '0.0.5'
        expected_max_required = '0.0.5b1'
        azext_metadata = {EXT_METADATA_MAXCLICOREVERSION: expected_max_required}
        with mock.patch('azure.cli.core.__version__', expected_cli_version):
            is_compatible, _, _, _, _ = ext_compat_with_cli(azext_metadata)
            self.assertFalse(is_compatible)

    def test_ext_compat_with_cli_require_ext_min_version(self):
        expected_cli_version = '0.0.5'
        expected_min_ext_required = '0.2.0'
        azext_metadata = {'name': 'myext', 'version': '0.1.0'}
        with mock.patch('azure.cli.core.__version__', expected_cli_version), \
            mock.patch('azure.cli.core.extension.EXTENSION_VERSION_REQUIREMENTS',
                       {'myext': {'minExtVersion': expected_min_ext_required}}):
            is_compatible, _, _, _, min_ext_required = ext_compat_with_cli(azext_metadata)
            self.assertFalse(is_compatible)
            self.assertEqual(min_ext_required, expected_min_ext_required)

    def test_list_available_extensions_preview_details(self):
        sample_index_extensions = {
            'ml': [{
                'metadata': {
                    'name': 'ml',
                    'summary': 'AzureMachineLearningWorkspaces Extension',
                    'version': '2.0.0a1',
                    'azext.isExperimental': True
                }
            }],
            'test_sample_extension1': [{
                'metadata': {
                    'name': 'test_sample_extension1',
                    'summary': 'my summary',
                    'version': '1.15.0',
                    'azext.isPreview': True,
                }
            }],
            'test_sample_extension2': [{
                'metadata': {
                    'name': 'test_sample_extension2',
                    'summary': 'my summary',
                    'version': '1.1.0b1'
                }
            }],
            'test_sample_extension3': [{
                'metadata': {
                    'name': 'test_sample_extension3',
                    'summary': 'my summary',
                    'version': '2.15.0'
                }
            }]
        }
        with mock.patch('azure.cli.core.extension.operations.get_index_extensions',
                        return_value=sample_index_extensions):
            res = list_available_extensions(cli_ctx=self.cmd.cli_ctx)
            self.assertIsInstance(res, list)
            self.assertEqual(len(res), len(sample_index_extensions))
            self.assertEqual(res[0]['name'], 'ml')
            self.assertEqual(res[0]['summary'], 'AzureMachineLearningWorkspaces Extension')
            self.assertEqual(res[0]['version'], '2.0.0a1')
            self.assertEqual(res[0]['preview'], True)
            self.assertEqual(res[0]['experimental'], False)
            self.assertEqual(res[1]['name'], 'test_sample_extension1')
            self.assertEqual(res[1]['version'], '1.15.0')
            self.assertEqual(res[1]['preview'], True)
            self.assertEqual(res[2]['name'], 'test_sample_extension2')
            self.assertEqual(res[2]['version'], '1.1.0b1')
            self.assertEqual(res[2]['preview'], True)
            self.assertEqual(res[3]['name'], 'test_sample_extension3')
            self.assertEqual(res[3]['version'], '2.15.0')
            self.assertEqual(res[3]['preview'], False)

    def test_add_list_show_preview_extension(self):
        test_ext_source = _get_test_data_file('ml-2.0.0a1-py3-none-any.whl')
        with mock.patch('azure.cli.core.extension.operations.logger') as mock_logger:
            add_extension(cmd=self.cmd, source=test_ext_source, allow_preview=True)
            call_args = mock_logger.warning.call_args
            self.assertEqual("The installed extension '%s' is in preview.", call_args[0][0])
            self.assertEqual("ml", call_args[0][1])
            self.assertEqual(mock_logger.warning.call_count, 1)
        ext = show_extension("ml")
        self.assertEqual(ext["name"], "ml")
        self.assertEqual(ext["version"], "2.0.0a1")
        remove_extension("ml")

    def test_add_stable_extension_by_default(self):
        test_ext_source = _get_test_data_file('extension_test_pkg-1.2.3-py3-none-any.whl')
        add_extension(cmd=self.cmd, source=test_ext_source)
        ext = show_extension("extension-test-pkg")
        self.assertEqual(ext["name"], "extension-test-pkg")
        self.assertEqual(ext["version"], "1.2.3")
        remove_extension("extension-test-pkg")

    def test_add_extension_without_preview(self):
        extension_name = "extension-test-pkg"
        extension1 = 'extension_test_pkg-1.0.0b1-py3-none-any.whl'
        extension2 = 'extension_test_pkg-1.2.3-py3-none-any.whl'
        extension3 = 'extension_test_pkg-1.4.1a1-py3-none-any.whl'

        mocked_index_data = {
            extension_name: [
                mock_ext(extension1, version='1.0.0b1', download_url=_get_test_data_file(extension1)),
                mock_ext(extension2, version='1.2.3', download_url=_get_test_data_file(extension2)),
                mock_ext(extension3, version='1.4.1a1', download_url=_get_test_data_file(extension3))
            ]
        }
        with mock.patch('azure.cli.core.extension._resolve.get_index_extensions',
                        return_value=mocked_index_data):
            add_extension(self.cmd, extension_name=extension_name, allow_preview=False)
            ext = show_extension(extension_name)
            self.assertEqual(ext['name'], extension_name)
            self.assertEqual(ext['version'], '1.2.3')
            remove_extension(extension_name)

    def test_add_extension_with_preview(self):
        extension_name = "extension-test-pkg"
        extension1 = 'extension_test_pkg-1.0.0b1-py3-none-any.whl'
        extension2 = 'extension_test_pkg-1.2.3-py3-none-any.whl'
        extension3 = 'extension_test_pkg-1.4.1a1-py3-none-any.whl'

        mocked_index_data = {
            extension_name: [
                mock_ext(extension1, version='1.0.0b1', download_url=_get_test_data_file(extension1)),
                mock_ext(extension2, version='1.2.3', download_url=_get_test_data_file(extension2)),
                mock_ext(extension3, version='1.4.1a1', download_url=_get_test_data_file(extension3))
            ]
        }

        with mock.patch('azure.cli.core.extension._resolve.get_index_extensions',
                        return_value=mocked_index_data):
            add_extension(self.cmd, extension_name=extension_name, allow_preview=True)
            ext = show_extension(extension_name)
            self.assertEqual(ext['name'], extension_name)
            self.assertEqual(ext['version'], '1.4.1a1')
            remove_extension(extension_name)

    def test_add_extension_preview_inavailable(self):
        extension_name = "extension-test-pkg"
        extension1 = 'extension_test_pkg-1.0.0b1-py3-none-any.whl'
        extension2 = 'extension_test_pkg-1.2.3-py3-none-any.whl'
        extension3 = 'extension_test_pkg-1.4.1a1-py3-none-any.whl'

        mocked_index_data = {
            extension_name: [
                mock_ext(extension1, version='1.0.0b1', download_url=_get_test_data_file(extension1)),
                mock_ext(extension3, version='1.4.1a1', download_url=_get_test_data_file(extension3))
            ]
        }
        from knack.util import CLIError
        with mock.patch('azure.cli.core.extension._resolve.get_index_extensions',
                        return_value=mocked_index_data):
            with self.assertRaisesRegex(CLIError, "No suitable stable version of 'extension-test-pkg' to install. Add `--allow-preview True` to try preview versions"):
                add_extension(cmd=self.cmd, extension_name=extension_name, allow_preview=False)

            with mock.patch('azure.cli.core.extension._resolve.logger') as mock_logger:
                add_extension(cmd=self.cmd, extension_name=extension_name)
                call_args = mock_logger.warning.call_args
                self.assertEqual("No stable version of '%s' to install. Preview versions allowed.", call_args[0][0])
                self.assertEqual(extension_name, call_args[0][1])
                self.assertEqual(mock_logger.warning.call_count, 1)
            ext = show_extension(extension_name)
            self.assertEqual(ext["name"], extension_name)
            self.assertEqual(ext["version"], "1.4.1a1")
            remove_extension(extension_name)

    def test_add_extension_with_later_preview(self):
        extension_name = "extension-test-pkg"
        extension1 = 'extension_test_pkg-1.0.0b1-py3-none-any.whl'
        extension2 = 'extension_test_pkg-1.2.3-py3-none-any.whl'
        extension3 = 'extension_test_pkg-1.4.1a1-py3-none-any.whl'

        mocked_index_data = {
            extension_name: [
                mock_ext(extension1, version='1.0.0b1', download_url=_get_test_data_file(extension1)),
                mock_ext(extension2, version='1.2.3', download_url=_get_test_data_file(extension2)),
                mock_ext(extension3, version='1.4.1a1', download_url=_get_test_data_file(extension3))
            ]
        }
        from knack.util import CLIError
        with mock.patch('azure.cli.core.extension._resolve.get_index_extensions',
                        return_value=mocked_index_data):
            with mock.patch('azure.cli.core.extension._resolve.logger') as mock_logger:
                add_extension(cmd=self.cmd, extension_name=extension_name)
                call_args = mock_logger.warning.call_args
                self.assertEqual("Extension '%s' has a later preview version to install, add `--allow-preview True` to try preview version.", call_args[0][0])
                self.assertEqual(extension_name, call_args[0][1])
                self.assertEqual(mock_logger.warning.call_count, 1)
            ext = show_extension(extension_name)
            self.assertEqual(ext["name"], extension_name)
            self.assertEqual(ext["version"], "1.2.3")
            remove_extension(extension_name)

    def test_update_extension_with_preview(self):
        extension_name = "extension-test-pkg"
        extension1 = 'extension_test_pkg-1.0.0b1-py3-none-any.whl'
        extension2 = 'extension_test_pkg-1.2.3-py3-none-any.whl'
        extension3 = 'extension_test_pkg-1.4.1a1-py3-none-any.whl'

        mocked_index_data = {
            extension_name: [
                mock_ext(extension1, version='1.0.0b1', download_url=_get_test_data_file(extension1)),
                mock_ext(extension2, version='1.2.3', download_url=_get_test_data_file(extension2)),
                mock_ext(extension3, version='1.4.1a1', download_url=_get_test_data_file(extension3))
            ]
        }
        with mock.patch('azure.cli.core.extension._resolve.get_index_extensions',
                        return_value=mocked_index_data):
            add_extension(self.cmd, extension_name=extension_name, allow_preview=False)
            ext = show_extension(extension_name)
            self.assertEqual(ext['name'], extension_name)
            self.assertEqual(ext['version'], '1.2.3')

            update_extension(self.cmd, extension_name, allow_preview=True)
            ext = show_extension(extension_name)
            self.assertEqual(ext['name'], extension_name)
            self.assertEqual(ext['version'], '1.4.1a1')

    @mock.patch('sys.stdin.isatty', return_value=True)
    def test_ext_dynamic_install_config_tty(self, _):
        from azure.cli.core.extension.dynamic_install import _get_extension_use_dynamic_install_config
        self.assertEqual(_get_extension_use_dynamic_install_config(None), 'yes_prompt')

    @mock.patch('sys.stdin.isatty', return_value=False)
    def test_ext_dynamic_install_config_no_tty(self, _):
        from azure.cli.core.extension.dynamic_install import _get_extension_use_dynamic_install_config
        self.assertEqual(_get_extension_use_dynamic_install_config(None), 'yes_without_prompt')


class TestWheelExtension(TestExtensionsBase):

    def test_wheel_get_all(self):
        _install_test_extension1()
        whl_exts = WheelExtension.get_all()
        self.assertEqual(len(whl_exts), 1)

    def test_wheel_version(self):
        _install_test_extension1()
        ext = get_extension(EXT_NAME)
        self.assertEqual(ext.version, EXT_VERSION)

    def test_wheel_type(self):
        _install_test_extension1()
        ext = get_extension(EXT_NAME)
        self.assertEqual(ext.ext_type, 'whl')

    def test_wheel_metadata1(self):
        _install_test_extension1()
        ext = get_extension(EXT_NAME)
        # There should be no exceptions and metadata should have some value
        self.assertTrue(ext.metadata)

    def test_wheel_metadata2(self):
        _install_test_extension2()
        ext = get_extension(EXT_NAME)
        # There should be no exceptions and metadata should have some value
        self.assertTrue(ext.metadata)
        # We check that we can retrieve any one of the az extension metadata values
        self.assertTrue(ext.metadata.get(EXT_METADATA_MINCLICOREVERSION))


class TestWheelSystemExtension(TestExtensionsBase):

    def test_wheel_get_all(self):
        _install_test_extension1(system=True)
        whl_exts = WheelExtension.get_all()
        self.assertEqual(len(whl_exts), 1)

    def test_wheel_user_system_extensions(self):
        _install_test_extension1()
        _install_test_extension3(system=True)
        whl_exts = WheelExtension.get_all()
        self.assertEqual(len(whl_exts), 2)

    def test_wheel_user_system_same_extension(self):
        _install_test_extension1()
        _install_test_extension1(system=True)
        self.assertNotEqual(build_extension_path(EXT_NAME), build_extension_path(EXT_NAME, system=True))
        actual = get_extension(EXT_NAME, ext_type=WheelExtension)
        self.assertEqual(actual.name, EXT_NAME)
        self.assertEqual(actual.path, build_extension_path(EXT_NAME))
        shutil.rmtree(self.ext_dir)
        actual = get_extension(EXT_NAME, ext_type=WheelExtension)
        self.assertEqual(actual.name, EXT_NAME)
        self.assertEqual(actual.path, build_extension_path(EXT_NAME, system=True))

    def test_wheel_version(self):
        _install_test_extension1(system=True)
        ext = get_extension(EXT_NAME)
        self.assertEqual(ext.version, EXT_VERSION)

    def test_wheel_type(self):
        _install_test_extension1(system=True)
        ext = get_extension(EXT_NAME)
        self.assertEqual(ext.ext_type, 'whl')

    def test_wheel_metadata1(self):
        _install_test_extension1(system=True)
        ext = get_extension(EXT_NAME)
        # There should be no exceptions and metadata should have some value
        self.assertTrue(ext.metadata)

    def test_wheel_metadata2(self):
        _install_test_extension2(system=True)
        ext = get_extension(EXT_NAME)
        # There should be no exceptions and metadata should have some value
        self.assertTrue(ext.metadata)
        # We check that we can retrieve any one of the az extension metadata values
        self.assertTrue(ext.metadata.get(EXT_METADATA_MINCLICOREVERSION))


if __name__ == '__main__':
    unittest.main()



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
#
# Code generated by aaz-dev-tools
# --------------------------------------------------------------------------------------------

import os
import tempfile

from azure.cli.testsdk import (ScenarioTest, ResourceGroupPreparer)
from azure.cli.testsdk.scenario_tests import AllowLargeResponse


class AksSafeguardsScenario(ScenarioTest):
    # TODO: add tests here
    @classmethod
    def generate_ssh_keys(cls):
        # If the `--ssh-key-value` option is not specified, the validator will try to read the ssh-key from the "~/.ssh" directory,
        # and if no key exists, it will call the method provided by azure-cli.core to generate one under the "~/.ssh" directory.
        # In order to avoid misuse of personal ssh-key during testing and the race condition that is prone to occur when key creation
        # is handled by azure-cli when performing test cases concurrently, we provide this function as a workround.

        # In the scenario of runner and AKS check-in pipeline, a temporary ssh-key will be generated in advance under the
        # "tests/latest/data/.ssh" sub-directory of the acs module in the cloned azure-cli repository when setting up the
        # environment. Each test case will read the ssh-key from a pre-generated file during execution, so there will be no
        # race conditions caused by concurrent reading and writing/creating of the same file.
        acs_base_dir = os.getenv("ACS_BASE_DIR", None)
        if acs_base_dir:
            pre_generated_ssh_key_path = os.path.join(
                acs_base_dir, "tests/latest/data/.ssh/id_rsa.pub")
            if os.path.exists(pre_generated_ssh_key_path):
                return pre_generated_ssh_key_path.replace('\\', '\\\\')

        # In the CLI check-in pipeline scenario, the following fake ssh-key will be used. Each test case will read the ssh-key from
        # a different temporary file during execution, so there will be no race conditions caused by concurrent reading and
        # writing/creating of the same file.
        TEST_SSH_KEY_PUB = "ssh-rsa AAAAB3NzaC1yc2EAAAADAQABAAACAQCbIg1guRHbI0lV11wWDt1r2cUdcNd27CJsg+SfgC7miZeubtwUhbsPdhMQsfDyhOWHq1+ZL0M+nJZV63d/1dhmhtgyOqejUwrPlzKhydsbrsdUor+JmNJDdW01v7BXHyuymT8G4s09jCasNOwiufbP/qp72ruu0bIA1nySsvlf9pCQAuFkAnVnf/rFhUlOkhtRpwcq8SUNY2zRHR/EKb/4NWY1JzR4sa3q2fWIJdrrX0DvLoa5g9bIEd4Df79ba7v+yiUBOS0zT2ll+z4g9izHK3EO5d8hL4jYxcjKs+wcslSYRWrascfscLgMlMGh0CdKeNTDjHpGPncaf3Z+FwwwjWeuiNBxv7bJo13/8B/098KlVDl4GZqsoBCEjPyJfV6hO0y/LkRGkk7oHWKgeWAfKtfLItRp00eZ4fcJNK9kCaSMmEugoZWcI7NGbZXzqFWqbpRI7NcDP9+WIQ+i9U5vqWsqd/zng4kbuAJ6UuKqIzB0upYrLShfQE3SAck8oaLhJqqq56VfDuASNpJKidV+zq27HfSBmbXnkR/5AK337dc3MXKJypoK/QPMLKUAP5XLPbs+NddJQV7EZXd29DLgp+fRIg3edpKdO7ZErWhv7d+3Kws+e1Y+ypmR2WIVSwVyBEUfgv2C8Ts9gnTF4pNcEY/S2aBicz5Ew2+jdyGNQQ== test@example.com\n"  # pylint: disable=line-too-long
        _, pathname = tempfile.mkstemp()
        with open(pathname, 'w') as key_file:
            key_file.write(TEST_SSH_KEY_PUB)
        return pathname.replace('\\', '\\\\')

    @AllowLargeResponse(size_kb=9999)
    @ResourceGroupPreparer(name_prefix='cli-', random_name_length=8, location="westus2")
    def test_aks_deployment_safeguards(self):
        self.kwargs.update({
            'aks_name': self.create_random_name(prefix='akssafeguards-', length=20),
            'ssh_key_value': self.generate_ssh_keys(),
            'vm_size': 'standard_a8_v2'
        })

        # Create AKS cluster
        aks_cluster = self.cmd('aks create -g {rg} -n {aks_name} --ssh-key-value={ssh_key_value} --node-vm-size {vm_size} --enable-addons azure-policy', checks=[
            self.check('name', '{aks_name}'),
            self.check('agentPoolProfiles[0].vmSize', '{vm_size}'),
        ]).get_output_in_json()

        # Enable safeguards with PSS level using -c/-g syntax
        self.cmd('aks safeguards create -c {aks_name} -g {rg} --level Warn --pss-level Baseline', checks=[
            self.check('properties.level', 'Warn'),
            self.check('properties.podSecurityStandardsLevel', 'Baseline'),
        ])

        # Get Safeguards using -c/-g syntax
        self.cmd('aks safeguards show -c {aks_name} -g {rg}', checks=[
            self.check('properties.level', 'Warn'),
            self.check('properties.podSecurityStandardsLevel', 'Baseline'),
            self.check('properties.excludedNamespaces', None),
        ])

        self.cmd('aks safeguards list -c {aks_name} -g {rg}', checks=[
            self.check('length(@)', 1),
            self.check('[0].properties.level', 'Warn'),
            self.check('[0].properties.podSecurityStandardsLevel', 'Baseline'),
            self.check('[0].properties.excludedNamespaces', None),
        ])

        # Change excluded namespaces and PSS level
        self.cmd('aks safeguards update -c {aks_name} -g {rg} --excluded-namespaces ns1 --pss-level Restricted', checks=[
            self.check('properties.excludedNamespaces[0]', 'ns1'),
            self.check('properties.podSecurityStandardsLevel', 'Restricted'),
        ])

        # Disable Safeguards using -c/-g syntax
        self.cmd('aks safeguards delete -c {aks_name} -g {rg} --yes')

        # delete the aks cluster
        self.cmd('aks delete -g {rg} -n {aks_name} --yes --no-wait')



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
#
# Code generated by aaz-dev-tools
# --------------------------------------------------------------------------------------------

# pylint: skip-file
# flake8: noqa

from azure.cli.core.aaz import *


@register_command(
    "eventhubs eventhub create",
)
class Create(AAZCommand):
    """Create a new Event Hub as a nested resource within a Namespace.
    """

    _aaz_info = {
        "version": "2024-05-01-preview",
        "resources": [
            ["mgmt-plane", "/subscriptions/{}/resourcegroups/{}/providers/microsoft.eventhub/namespaces/{}/eventhubs/{}", "2024-05-01-preview"],
        ]
    }

    def _handler(self, command_args):
        super()._handler(command_args)
        self._execute_operations()
        return self._output()

    _args_schema = None

    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        if cls._args_schema is not None:
            return cls._args_schema
        cls._args_schema = super()._build_arguments_schema(*args, **kwargs)

        # define Arg Group ""

        _args_schema = cls._args_schema
        _args_schema.event_hub_name = AAZStrArg(
            options=["-n", "--name", "--event-hub-name"],
            help="The Event Hub name",
            required=True,
            fmt=AAZStrArgFormat(
                max_length=256,
                min_length=1,
            ),
        )
        _args_schema.namespace_name = AAZStrArg(
            options=["--namespace-name"],
            help="The Namespace name",
            required=True,
            fmt=AAZStrArgFormat(
                pattern="^[a-zA-Z][a-zA-Z0-9-]{4,48}[a-zA-Z0-9]$",
                max_length=50,
                min_length=6,
            ),
        )
        _args_schema.resource_group = AAZResourceGroupNameArg(
            required=True,
        )

        # define Arg Group "CaptureDescription"

        _args_schema = cls._args_schema
        _args_schema.destination_name = AAZStrArg(
            options=["--destination-name"],
            arg_group="CaptureDescription",
            help="Name for capture destination",
        )
        _args_schema.archive_name_format = AAZStrArg(
            options=["--archive-name-format"],
            arg_group="CaptureDescription",
            help="Blob naming convention for archive, e.g. {Namespace}/{EventHub}/{PartitionId}/{Year}/{Month}/{Day}/{Hour}/{Minute}/{Second}. Here all the parameters (Namespace,EventHub .. etc) are mandatory irrespective of order",
        )
        _args_schema.blob_container = AAZStrArg(
            options=["--blob-container"],
            arg_group="CaptureDescription",
            help="Blob container Name",
        )
        _args_schema.storage_account = AAZStrArg(
            options=["--storage-account"],
            arg_group="CaptureDescription",
            help="Resource id of the storage account to be used to create the blobs",
        )
        _args_schema.enable_capture = AAZBoolArg(
            options=["--enable-capture"],
            arg_group="CaptureDescription",
            help="A value that indicates whether capture description is enabled.",
        )
        _args_schema.encoding = AAZStrArg(
            options=["--encoding"],
            arg_group="CaptureDescription",
            help="Enumerates the possible values for the encoding format of capture description. Note: 'AvroDeflate' will be deprecated in New API Version",
            enum={"Avro": "Avro", "AvroDeflate": "AvroDeflate"},
        )
        _args_schema.capture_interval = AAZIntArg(
            options=["--capture-interval"],
            arg_group="CaptureDescription",
            help="The time window allows you to set the frequency with which the capture to Azure Blobs will happen, value should between 60 to 900 seconds",
        )
        _args_schema.capture_size_limit = AAZIntArg(
            options=["--capture-size-limit"],
            arg_group="CaptureDescription",
            help="The size window defines the amount of data built up in your Event Hub before an capture operation, value should be between 10485760 to 524288000 bytes",
        )
        _args_schema.skip_empty_archives = AAZBoolArg(
            options=["--skip-empty-archives"],
            arg_group="CaptureDescription",
            help="A value that indicates whether to Skip Empty Archives",
        )

        # define Arg Group "Destination"

        _args_schema = cls._args_schema
        _args_schema.identity = AAZObjectArg(
            options=["--identity"],
            arg_group="Destination",
            help="A value that indicates whether capture description is enabled.",
        )

        identity = cls._args_schema.identity
        identity.type = AAZStrArg(
            options=["type"],
            help="Type of Azure Active Directory Managed Identity.",
            enum={"SystemAssigned": "SystemAssigned", "UserAssigned": "UserAssigned"},
        )
        identity.user_assigned_identity = AAZStrArg(
            options=["user-assigned-identity"],
            help="ARM ID of Managed User Identity. This property is required is the type is UserAssignedIdentity. If type is SystemAssigned, then the System Assigned Identity Associated with the namespace will be used.",
        )

        # define Arg Group "MessageTimestampDescription"

        _args_schema = cls._args_schema
        _args_schema.timestamp_type = AAZStrArg(
            options=["--timestamp-type"],
            arg_group="MessageTimestampDescription",
            help="Denotes the type of timestamp the message will hold.Two types of timestamp types - \"AppendTime\" and \"CreateTime\". AppendTime refers the time in which message got appended inside broker log. CreateTime refers to the time in which the message was generated on source side and producers can set this timestamp while sending the message. Default value is AppendTime. If you are using AMQP protocol, CreateTime equals AppendTime and its behavior remains the same.",
            enum={"Create": "Create", "LogAppend": "LogAppend"},
        )

        # define Arg Group "Properties"

        _args_schema = cls._args_schema
        _args_schema.message_retention_in_days = AAZIntArg(
            options=["--message-retention-in-days"],
            arg_group="Properties",
            help="Number of days to retain the events for this Event Hub, value should be 1 to 7 days",
            fmt=AAZIntArgFormat(
                minimum=1,
            ),
        )
        _args_schema.partition_count = AAZIntArg(
            options=["--partition-count"],
            arg_group="Properties",
            help="Number of partitions created for the Event Hub, allowed values are from 1 to 32 partitions.",
            fmt=AAZIntArgFormat(
                minimum=1,
            ),
        )
        _args_schema.status = AAZStrArg(
            options=["--status"],
            arg_group="Properties",
            help="Enumerates the possible values for the status of the Event Hub.",
            enum={"Active": "Active", "Creating": "Creating", "Deleting": "Deleting", "Disabled": "Disabled", "ReceiveDisabled": "ReceiveDisabled", "Renaming": "Renaming", "Restoring": "Restoring", "SendDisabled": "SendDisabled", "Unknown": "Unknown"},
        )
        _args_schema.user_metadata = AAZStrArg(
            options=["--user-metadata"],
            arg_group="Properties",
            help="Gets and Sets Metadata of User.",
        )

        # define Arg Group "RetentionDescription"

        _args_schema = cls._args_schema
        _args_schema.cleanup_policy = AAZStrArg(
            options=["--cleanup-policy"],
            arg_group="RetentionDescription",
            help="Enumerates the possible values for cleanup policy",
            enum={"Compact": "Compact", "Delete": "Delete", "DeleteOrCompact": "DeleteOrCompact"},
        )
        _args_schema.min_compaction_lag_time_in_minutes = AAZIntArg(
            options=["--min-lag", "--min-compaction-lag-in-mins"],
            arg_group="RetentionDescription",
            help="The minimum time a message will remain ineligible for compaction in the log. This value is used when cleanupPolicy is Compact or DeleteOrCompact.",
        )
        _args_schema.retention_time_in_hours = AAZIntArg(
            options=["--retention-time", "--retention-time-in-hours"],
            arg_group="RetentionDescription",
            help="Number of hours to retain the events for this Event Hub. This value is only used when cleanupPolicy is Delete. If cleanupPolicy is Compaction the returned value of this property is Long.MaxValue",
        )
        _args_schema.tombstone_retention_time_in_hours = AAZIntArg(
            options=["-t", "--tombstone-retention-time-in-hours"],
            arg_group="RetentionDescription",
            help="Number of hours to retain the tombstone markers of a compacted Event Hub. This value is only used when cleanupPolicy is Compaction. Consumer must complete reading the tombstone marker within this specified amount of time if consumer begins from starting offset to ensure they get a valid snapshot for the specific key described by the tombstone marker within the compacted Event Hub",
        )
        return cls._args_schema

    def _execute_operations(self):
        self.pre_operations()
        self.EventHubsCreateOrUpdate(ctx=self.ctx)()
        self.post_operations()

    @register_callback
    def pre_operations(self):
        pass

    @register_callback
    def post_operations(self):
        pass

    def _output(self, *args, **kwargs):
        result = self.deserialize_output(self.ctx.vars.instance, client_flatten=True)
        return result

    class EventHubsCreateOrUpdate(AAZHttpOperation):
        CLIENT_TYPE = "MgmtClient"

        def __call__(self, *args, **kwargs):
            request = self.make_request()
            session = self.client.send_request(request=request, stream=False, **kwargs)
            if session.http_response.status_code in [200]:
                return self.on_200(session)

            return self.on_error(session.http_response)

        @property
        def url(self):
            return self.client.format_url(
                "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.EventHub/namespaces/{namespaceName}/eventhubs/{eventHubName}",
                **self.url_parameters
            )

        @property
        def method(self):
            return "PUT"

        @property
        def error_format(self):
            return "MgmtErrorFormat"

        @property
        def url_parameters(self):
            parameters = {
                **self.serialize_url_param(
                    "eventHubName", self.ctx.args.event_hub_name,
                    required=True,
                ),
                **self.serialize_url_param(
                    "namespaceName", self.ctx.args.namespace_name,
                    required=True,
                ),
                **self.serialize_url_param(
                    "resourceGroupName", self.ctx.args.resource_group,
                    required=True,
                ),
                **self.serialize_url_param(
                    "subscriptionId", self.ctx.subscription_id,
                    required=True,
                ),
            }
            return parameters

        @property
        def query_parameters(self):
            parameters = {
                **self.serialize_query_param(
                    "api-version", "2024-05-01-preview",
                    required=True,
                ),
            }
            return parameters

        @property
        def header_parameters(self):
            parameters = {
                **self.serialize_header_param(
                    "Content-Type", "application/json",
                ),
                **self.serialize_header_param(
                    "Accept", "application/json",
                ),
            }
            return parameters

        @property
        def content(self):
            _content_value, _builder = self.new_content_builder(
                self.ctx.args,
                typ=AAZObjectType,
                typ_kwargs={"flags": {"required": True, "client_flatten": True}}
            )
            _builder.set_prop("properties", AAZObjectType, typ_kwargs={"flags": {"client_flatten": True}})

            properties = _builder.get(".properties")
            if properties is not None:
                properties.set_prop("captureDescription", AAZObjectType)
                properties.set_prop("messageRetentionInDays", AAZIntType, ".message_retention_in_days")
                properties.set_prop("messageTimestampDescription", AAZObjectType)
                properties.set_prop("partitionCount", AAZIntType, ".partition_count")
                properties.set_prop("retentionDescription", AAZObjectType)
                properties.set_prop("status", AAZStrType, ".status")
                properties.set_prop("userMetadata", AAZStrType, ".user_metadata")

            capture_description = _builder.get(".properties.captureDescription")
            if capture_description is not None:
                capture_description.set_prop("destination", AAZObjectType)
                capture_description.set_prop("enabled", AAZBoolType, ".enable_capture")
                capture_description.set_prop("encoding", AAZStrType, ".encoding")
                capture_description.set_prop("intervalInSeconds", AAZIntType, ".capture_interval")
                capture_description.set_prop("sizeLimitInBytes", AAZIntType, ".capture_size_limit")
                capture_description.set_prop("skipEmptyArchives", AAZBoolType, ".skip_empty_archives")

            destination = _builder.get(".properties.captureDescription.destination")
            if destination is not None:
                destination.set_prop("identity", AAZObjectType, ".identity")
                destination.set_prop("name", AAZStrType, ".destination_name")
                destination.set_prop("properties", AAZObjectType, typ_kwargs={"flags": {"client_flatten": True}})

            identity = _builder.get(".properties.captureDescription.destination.identity")
            if identity is not None:
                identity.set_prop("type", AAZStrType, ".type")
                identity.set_prop("userAssignedIdentity", AAZStrType, ".user_assigned_identity")

            properties = _builder.get(".properties.captureDescription.destination.properties")
            if properties is not None:
                properties.set_prop("archiveNameFormat", AAZStrType, ".archive_name_format")
                properties.set_prop("blobContainer", AAZStrType, ".blob_container")
                properties.set_prop("storageAccountResourceId", AAZStrType, ".storage_account")

            message_timestamp_description = _builder.get(".properties.messageTimestampDescription")
            if message_timestamp_description is not None:
                message_timestamp_description.set_prop("timestampType", AAZStrType, ".timestamp_type")

            retention_description = _builder.get(".properties.retentionDescription")
            if retention_description is not None:
                retention_description.set_prop("cleanupPolicy", AAZStrType, ".cleanup_policy")
                retention_description.set_prop("minCompactionLagTimeInMinutes", AAZIntType, ".min_compaction_lag_time_in_minutes")
                retention_description.set_prop("retentionTimeInHours", AAZIntType, ".retention_time_in_hours")
                retention_description.set_prop("tombstoneRetentionTimeInHours", AAZIntType, ".tombstone_retention_time_in_hours")

            return self.serialize_content(_content_value)

        def on_200(self, session):
            data = self.deserialize_http_content(session)
            self.ctx.set_var(
                "instance",
                data,
                schema_builder=self._build_schema_on_200
            )

        _schema_on_200 = None

        @classmethod
        def _build_schema_on_200(cls):
            if cls._schema_on_200 is not None:
                return cls._schema_on_200

            cls._schema_on_200 = AAZObjectType()

            _schema_on_200 = cls._schema_on_200
            _schema_on_200.id = AAZStrType(
                flags={"read_only": True},
            )
            _schema_on_200.location = AAZStrType(
                flags={"read_only": True},
            )
            _schema_on_200.name = AAZStrType(
                flags={"read_only": True},
            )
            _schema_on_200.properties = AAZObjectType(
                flags={"client_flatten": True},
            )
            _schema_on_200.system_data = AAZObjectType(
                serialized_name="systemData",
                flags={"read_only": True},
            )
            _schema_on_200.type = AAZStrType(
                flags={"read_only": True},
            )

            properties = cls._schema_on_200.properties
            properties.capture_description = AAZObjectType(
                serialized_name="captureDescription",
            )
            properties.created_at = AAZStrType(
                serialized_name="createdAt",
                flags={"read_only": True},
            )
            properties.identifier = AAZStrType(
                flags={"read_only": True},
            )
            properties.message_retention_in_days = AAZIntType(
                serialized_name="messageRetentionInDays",
            )
            properties.message_timestamp_description = AAZObjectType(
                serialized_name="messageTimestampDescription",
            )
            properties.partition_count = AAZIntType(
                serialized_name="partitionCount",
            )
            properties.partition_ids = AAZListType(
                serialized_name="partitionIds",
                flags={"read_only": True},
            )
            properties.retention_description = AAZObjectType(
                serialized_name="retentionDescription",
            )
            properties.status = AAZStrType()
            properties.updated_at = AAZStrType(
                serialized_name="updatedAt",
                flags={"read_only": True},
            )
            properties.user_metadata = AAZStrType(
                serialized_name="userMetadata",
            )

            capture_description = cls._schema_on_200.properties.capture_description
            capture_description.destination = AAZObjectType()
            capture_description.enabled = AAZBoolType()
            capture_description.encoding = AAZStrType()
            capture_description.interval_in_seconds = AAZIntType(
                serialized_name="intervalInSeconds",
            )
            capture_description.size_limit_in_bytes = AAZIntType(
                serialized_name="sizeLimitInBytes",
            )
            capture_description.skip_empty_archives = AAZBoolType(
                serialized_name="skipEmptyArchives",
            )

            destination = cls._schema_on_200.properties.capture_description.destination
            destination.identity = AAZObjectType()
            destination.name = AAZStrType()
            destination.properties = AAZObjectType(
                flags={"client_flatten": True},
            )

            identity = cls._schema_on_200.properties.capture_description.destination.identity
            identity.type = AAZStrType()
            identity.user_assigned_identity = AAZStrType(
                serialized_name="userAssignedIdentity",
            )

            properties = cls._schema_on_200.properties.capture_description.destination.properties
            properties.archive_name_format = AAZStrType(
                serialized_name="archiveNameFormat",
            )
            properties.blob_container = AAZStrType(
                serialized_name="blobContainer",
            )
            properties.data_lake_account_name = AAZStrType(
                serialized_name="dataLakeAccountName",
            )
            properties.data_lake_folder_path = AAZStrType(
                serialized_name="dataLakeFolderPath",
            )
            properties.data_lake_subscription_id = AAZStrType(
                serialized_name="dataLakeSubscriptionId",
            )
            properties.storage_account_resource_id = AAZStrType(
                serialized_name="storageAccountResourceId",
            )

            message_timestamp_description = cls._schema_on_200.properties.message_timestamp_description
            message_timestamp_description.timestamp_type = AAZStrType(
                serialized_name="timestampType",
            )

            partition_ids = cls._schema_on_200.properties.partition_ids
            partition_ids.Element = AAZStrType()

            retention_description = cls._schema_on_200.properties.retention_description
            retention_description.cleanup_policy = AAZStrType(
                serialized_name="cleanupPolicy",
            )
            retention_description.min_compaction_lag_time_in_minutes = AAZIntType(
                serialized_name="minCompactionLagTimeInMinutes",
            )
            retention_description.retention_time_in_hours = AAZIntType(
                serialized_name="retentionTimeInHours",
            )
            retention_description.tombstone_retention_time_in_hours = AAZIntType(
                serialized_name="tombstoneRetentionTimeInHours",
            )

            system_data = cls._schema_on_200.system_data
            system_data.created_at = AAZStrType(
                serialized_name="createdAt",
            )
            system_data.created_by = AAZStrType(
                serialized_name="createdBy",
            )
            system_data.created_by_type = AAZStrType(
                serialized_name="createdByType",
            )
            system_data.last_modified_at = AAZStrType(
                serialized_name="lastModifiedAt",
            )
            system_data.last_modified_by = AAZStrType(
                serialized_name="lastModifiedBy",
            )
            system_data.last_modified_by_type = AAZStrType(
                serialized_name="lastModifiedByType",
            )

            return cls._schema_on_200


class _CreateHelper:
    """Helper class for Create"""


__all__ = ["Create"]



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------


from azure.cli.testsdk.scenario_tests import AllowLargeResponse
from azure.cli.testsdk import (ScenarioTest, ResourceGroupPreparer, JMESPathCheck)

from .common import TEST_LOCATION
from .utils import prepare_containerapp_env_for_app_e2e_tests


# flake8: noqa
# noqa
# pylint: skip-file


class ContainerAppAuthTest(ScenarioTest):
    def __init__(self, *arg, **kwargs):
        super().__init__(*arg, random_config_dir=True, **kwargs)

    @AllowLargeResponse(8192)
    @ResourceGroupPreparer(location=TEST_LOCATION)
    def test_containerapp_auth_e2e(self, resource_group):
        self.cmd('configure --defaults location={}'.format(TEST_LOCATION))
        app = self.create_random_name(prefix='containerapp-auth', length=24)

        env_id = prepare_containerapp_env_for_app_e2e_tests(self)

        self.cmd('containerapp create -g {} -n {} --environment {} --image mcr.microsoft.com/k8se/quickstart:latest --ingress external --target-port 80'.format(resource_group, app, env_id))

        client_id = 'abc123'
        test_secret = 'abc123'
        issuer = 'https://sts.windows.net/54826b22-38d6-4fb2-bad9-b7983a3e9c5a/'

        self.cmd(
            'containerapp auth microsoft update -g {} --name {} --client-id {} --client-secret {} --issuer {} --yes'
            .format(resource_group, app, client_id, test_secret, issuer), checks=[
                JMESPathCheck('registration.clientId', client_id),
                JMESPathCheck('registration.clientSecretSettingName',
                              "microsoft-provider-authentication-secret"),
                JMESPathCheck('registration.openIdIssuer', issuer),
            ])

        self.cmd('containerapp auth microsoft show -g {} --name {}'.format(resource_group, app), checks=[
                JMESPathCheck('registration.clientId', client_id),
                JMESPathCheck('registration.clientSecretSettingName',
                              "microsoft-provider-authentication-secret"),
                JMESPathCheck('registration.openIdIssuer', issuer),
            ])

        self.cmd('containerapp secret list -g {} -n {}'.format(resource_group, app), checks=[
            JMESPathCheck('length(@)', 1),
            JMESPathCheck('[0].name', "microsoft-provider-authentication-secret")
        ])

        self.cmd('containerapp auth show -g {} -n {}'.format(resource_group, app), checks=[
            JMESPathCheck('identityProviders.azureActiveDirectory.registration.clientId', client_id),
            JMESPathCheck('identityProviders.azureActiveDirectory.registration.clientSecretSettingName',
                          "microsoft-provider-authentication-secret"),
            JMESPathCheck('identityProviders.azureActiveDirectory.registration.openIdIssuer', issuer),
        ])

        sas_url = "sasurl"
        self.cmd('containerapp auth update  -g {} -n {} --unauthenticated-client-action AllowAnonymous --token-store true --sas-url-secret {} --yes'.format(resource_group, app, sas_url), checks=[
            JMESPathCheck('name', "current"),
            JMESPathCheck('properties.globalValidation.unauthenticatedClientAction', "AllowAnonymous"),
            JMESPathCheck('properties.identityProviders.azureActiveDirectory.registration.clientId', client_id),
            JMESPathCheck('properties.identityProviders.azureActiveDirectory.registration.clientSecretSettingName',
                          "microsoft-provider-authentication-secret"),
            JMESPathCheck('properties.identityProviders.azureActiveDirectory.registration.openIdIssuer', issuer),
            JMESPathCheck('properties.login.tokenStore.enabled', True),
            JMESPathCheck('properties.login.tokenStore.azureBlobStorage.sasUrlSettingName',
                          "blob-storage-token-store-sasurl-secret"),
        ])

        self.cmd("az containerapp secret list --resource-group {} --name {}".format(resource_group, app), checks=[
            JMESPathCheck('length(@)', 2),
        ])

        self.cmd('containerapp auth show -g {} -n {}'.format(resource_group, app), checks=[
            JMESPathCheck('globalValidation.unauthenticatedClientAction', "AllowAnonymous"),
            JMESPathCheck('identityProviders.azureActiveDirectory.registration.clientId', client_id),
            JMESPathCheck('identityProviders.azureActiveDirectory.registration.clientSecretSettingName',
                          "microsoft-provider-authentication-secret"),
            JMESPathCheck('identityProviders.azureActiveDirectory.registration.openIdIssuer', issuer),
            JMESPathCheck('login.tokenStore.enabled', True),
            JMESPathCheck('login.tokenStore.azureBlobStorage.sasUrlSettingName',
                          "blob-storage-token-store-sasurl-secret"),
        ])
        login_paramters = 'identityProviders.azureActiveDirectory.login.loginParameters=[a,scope=openid offline_access api://<back-end-client-id>/user_impersonation]'
        self.cmd("containerapp auth update -g {} -n {} --proxy-convention Standard --redirect-provider Facebook --unauthenticated-client-action AllowAnonymous --set '{}'".format(
                resource_group, app, login_paramters), checks=[
                JMESPathCheck('name', 'current'),
                JMESPathCheck('properties.httpSettings.forwardProxy.convention', 'Standard'),
                JMESPathCheck('properties.globalValidation.redirectToProvider', 'Facebook'),
                JMESPathCheck('properties.globalValidation.unauthenticatedClientAction', 'AllowAnonymous'),
                JMESPathCheck('properties.identityProviders.azureActiveDirectory.registration.clientId', client_id),
                JMESPathCheck('properties.identityProviders.azureActiveDirectory.registration.clientSecretSettingName', "microsoft-provider-authentication-secret"),
                JMESPathCheck('properties.identityProviders.azureActiveDirectory.registration.openIdIssuer', issuer),
                JMESPathCheck('length(properties.identityProviders.azureActiveDirectory.login.loginParameters)', 2),
                JMESPathCheck('properties.identityProviders.azureActiveDirectory.login.loginParameters[0]', "a"),
                JMESPathCheck('properties.identityProviders.azureActiveDirectory.login.loginParameters[1]', "scope=openid offline_access api://<back-end-client-id>/user_impersonation"),
        ])

        self.cmd('containerapp show  -g {} -n {}'.format(resource_group, app), checks=[
            JMESPathCheck('properties.provisioningState', "Succeeded")
        ])

        self.cmd('containerapp auth microsoft update -g {} --name {} --client-id {} --client-secret {} --client-secret-name {} --san a --thumbprint a --issuer {} --allowed-audiences a --tenant-id a --yes'
                 .format(resource_group, app, client_id, test_secret, test_secret, issuer), expect_failure=True)

        self.cmd('containerapp auth update -g {} --name {} --config-file-path a --custom-host-header b --set a --custom-proto-header c --enabled true --excluded-paths b --proxy-convention NoProxy --redirect-provider c --require-https false --runtime-version 0.0.0 '.format(resource_group, app), expect_failure=True)

    @AllowLargeResponse(8192)
    @ResourceGroupPreparer(location=TEST_LOCATION)
    def test_containerapp_auth_facebook_e2e(self, resource_group):
        self.cmd('configure --defaults location={}'.format(TEST_LOCATION))
        app = self.create_random_name(prefix='containerapp-auth', length=24)

        env_id = prepare_containerapp_env_for_app_e2e_tests(self)

        self.cmd(
            'containerapp create -g {} -n {} --environment {} --image mcr.microsoft.com/k8se/quickstart:latest --ingress external --target-port 80'.format(
                resource_group, app, env_id))

        app_id = 'abc123'
        test_secret = 'abc123'
        issuer = 'https://sts.windows.net/54826b22-38d6-4fb2-bad9-b7983a3e9c5a/'

        self.cmd(
            'containerapp auth facebook update  -g {} --name {} --app-id {} --app-secret {} --graph-api-version 0.0.2 --scopes cc --yes'
            .format(resource_group, app, app_id, test_secret, issuer).format(resource_group, app), checks=[
                JMESPathCheck('graphApiVersion', "0.0.2"),
                JMESPathCheck('login.scopes[0]', "cc"),
                JMESPathCheck('registration.appId', app_id),
                JMESPathCheck('registration.appSecretSettingName', "facebook-provider-authentication-secret"),
            ])
        self.cmd(
            'containerapp auth facebook show  -g {} --name {}'
            .format(resource_group, app), checks=[
                JMESPathCheck('graphApiVersion', "0.0.2"),
                JMESPathCheck('login.scopes[0]', "cc"),
                JMESPathCheck('registration.appId', app_id),
                JMESPathCheck('registration.appSecretSettingName', "facebook-provider-authentication-secret"),
            ])

        self.cmd(
            'containerapp auth facebook update  -g {} --name {} --app-id {} --app-secret-name {} --graph-api-version 0.0.2 --scopes cc --yes'
            .format(resource_group, app, app_id, test_secret, issuer).format(resource_group, app), expect_failure=True)

    @AllowLargeResponse(8192)
    @ResourceGroupPreparer(location=TEST_LOCATION)
    def test_containerapp_auth_github_e2e(self, resource_group):
        self.cmd('configure --defaults location={}'.format(TEST_LOCATION))
        app = self.create_random_name(prefix='containerapp-auth', length=24)

        env_id = prepare_containerapp_env_for_app_e2e_tests(self)

        self.cmd(
            'containerapp create -g {} -n {} --environment {} --image mcr.microsoft.com/k8se/quickstart:latest --ingress external --target-port 80'.format(
                resource_group, app, env_id))

        client_id = 'abc123'
        test_secret = 'abc123'
        # github
        self.cmd(
            'containerapp auth github update -g {} --name {} --client-id {} --client-secret {} --yes'
            .format(resource_group, app, client_id, test_secret), checks=[
                JMESPathCheck('registration.clientId', client_id),
                JMESPathCheck('registration.clientSecretSettingName',
                              "github-provider-authentication-secret"),
            ])

        self.cmd('containerapp auth github show -g {} --name {}'.format(resource_group, app), checks=[
            JMESPathCheck('registration.clientId', client_id),
            JMESPathCheck('registration.clientSecretSettingName',
                          "github-provider-authentication-secret"),
        ])

        # google
        app = self.create_random_name(prefix='containerapp-auth', length=24)

        self.cmd(
            'containerapp create -g {} -n {} --environment {} --image mcr.microsoft.com/k8se/quickstart:latest --ingress external --target-port 80'.format(
                resource_group, app, env_id))

        self.cmd(
            'containerapp auth google update -g {} --name {} --client-id {} --client-secret {} --yes'
            .format(resource_group, app, client_id, test_secret), checks=[
                JMESPathCheck('registration.clientId', client_id),
                JMESPathCheck('registration.clientSecretSettingName',
                              "google-provider-authentication-secret"),
            ])

        self.cmd('containerapp auth google show -g {} --name {}'.format(resource_group, app), checks=[
            JMESPathCheck('registration.clientId', client_id),
            JMESPathCheck('registration.clientSecretSettingName',
                          "google-provider-authentication-secret"),
        ])
        # apple
        app = self.create_random_name(prefix='containerapp-auth', length=24)

        self.cmd(
            'containerapp create -g {} -n {} --environment {} --image mcr.microsoft.com/k8se/quickstart:latest --ingress external --target-port 80'.format(
                resource_group, app, env_id))

        self.cmd(
            'containerapp auth apple update -g {} --name {} --client-id {} --client-secret {} --yes'
            .format(resource_group, app, client_id, test_secret), checks=[
                JMESPathCheck('registration.clientId', client_id),
                JMESPathCheck('registration.clientSecretSettingName',
                              "apple-provider-authentication-secret"),
            ])

        self.cmd('containerapp auth apple show -g {} --name {}'.format(resource_group, app), checks=[
            JMESPathCheck('registration.clientId', client_id),
            JMESPathCheck('registration.clientSecretSettingName',
                          "apple-provider-authentication-secret"),
        ])
        # twitter
        app = self.create_random_name(prefix='containerapp-auth', length=24)

        self.cmd(
            'containerapp create -g {} -n {} --environment {} --image mcr.microsoft.com/k8se/quickstart:latest --ingress external --target-port 80'.format(
                resource_group, app, env_id))

        self.cmd(
            'containerapp auth twitter update -g {} --name {} --consumer-key {} --consumer-secret {} --yes'
            .format(resource_group, app, client_id, test_secret), checks=[
                JMESPathCheck('registration.consumerKey', client_id),
                JMESPathCheck('registration.consumerSecretSettingName',
                              "twitter-provider-authentication-secret"),
            ])

        self.cmd('containerapp auth twitter show -g {} --name {}'.format(resource_group, app), checks=[
            JMESPathCheck('registration.consumerKey', client_id),
            JMESPathCheck('registration.consumerSecretSettingName',
                          "twitter-provider-authentication-secret"),
        ])

        self.cmd(
            'containerapp auth twitter update -g {} --name {} --consumer-key {} --consumer-secret-name {} --yes'
            .format(resource_group, app, client_id, test_secret), expect_failure=True)


    @AllowLargeResponse(8192)
    @ResourceGroupPreparer(location=TEST_LOCATION)
    def test_containerapp_auth_openid_connect_e2e(self, resource_group):
        self.cmd('configure --defaults location={}'.format(TEST_LOCATION))
        app = self.create_random_name(prefix='containerapp-auth', length=24)

        env_id = prepare_containerapp_env_for_app_e2e_tests(self)

        self.cmd(
            'containerapp create -g {} -n {} --environment {} --image mcr.microsoft.com/k8se/quickstart:latest --ingress external --target-port 80'.format(
                resource_group, app, env_id))

        client_id = 'abc123'
        test_secret = 'abc123'

        # openid-connect
        app = self.create_random_name(prefix='containerapp-auth', length=24)
        self.cmd(
            'containerapp create -g {} -n {} --environment {} --image mcr.microsoft.com/k8se/quickstart:latest --ingress external --target-port 80'.format(
                resource_group, app, env_id))

        provider_name = "customopenid"
        configuration = "test-openid-configuration"
        self.cmd(
            'containerapp auth openid-connect add -g {} --name {} --client-id {} --provider-name {} --openid-configuration {} --scope sc --yes'
            .format(resource_group, app, client_id, provider_name, configuration), checks=[
            JMESPathCheck('registration.clientId', client_id),
            JMESPathCheck('registration.openIdConnectConfiguration.wellKnownOpenIdConfiguration', configuration),
        ])

        self.cmd('containerapp auth openid-connect show -g {} --name {} --provider-name {}'.format(resource_group, app, provider_name), expect_failure=False)
        self.cmd( 'containerapp auth openid-connect remove -g {} --name {} --provider-name {} --yes'.format(resource_group, app, provider_name), expect_failure=False)
        self.cmd('containerapp auth openid-connect update -g {} --name {} --client-id {} --client-secret {} --client-secret-name {} --provider-name {} --openid-configuration {} --scope sc --yes'
            .format(resource_group, app, client_id, test_secret, test_secret, provider_name, configuration), expect_failure=True)



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

import os
import time

from azure.mgmt.core.tools import parse_resource_id

from azure.cli.testsdk.scenario_tests import AllowLargeResponse
from azure.cli.testsdk import (ScenarioTest, ResourceGroupPreparer, JMESPathCheck, LogAnalyticsWorkspacePreparer)

from azure.cli.command_modules.containerapp.tests.latest.common import TEST_LOCATION
from .utils import prepare_containerapp_env_for_app_e2e_tests

TEST_DIR = os.path.abspath(os.path.join(os.path.abspath(__file__), '..'))
# flake8: noqa
# noqa
# pylint: skip-file


class ContainerAppJobsSecretsOperationsTest(ScenarioTest):
    def __init__(self, *arg, **kwargs):
        super().__init__(*arg, random_config_dir=True, **kwargs)

    @AllowLargeResponse(8192)
    @ResourceGroupPreparer(location="northcentralus")
    # test for CRUD operations on Container App Job resource with trigger type as manual
    def test_containerapp_manualjob_withsecret_crudoperations_e2e(self, resource_group):

        self.cmd('configure --defaults location={}'.format(TEST_LOCATION))

        job = self.create_random_name(prefix='job1', length=24)

        env_id = prepare_containerapp_env_for_app_e2e_tests(self)
        env_rg = parse_resource_id(env_id).get('resource_group')
        env_name = parse_resource_id(env_id).get('name')

        # create a container app environment for a Container App Job resource
        self.cmd('containerapp env show -n {} -g {}'.format(env_name, env_rg), checks=[
            JMESPathCheck('name', env_name)
        ])

        ## test for CRUD operations on Container App Job resource with trigger type as manual
        # create a Container App Job resource with trigger type as manual
        self.cmd("az containerapp job create --resource-group {} --name {} --environment {} --secrets 'testsecret=testsecretvalue' --replica-timeout 200 --replica-retry-limit 2 --trigger-type manual --parallelism 1 --replica-completion-count 1 --image mcr.microsoft.com/k8se/quickstart-jobs:latest --cpu '0.25' --memory '0.5Gi'".format(resource_group, job, env_id))

        # verify the container app job resource contains the secret
        self.cmd("az containerapp job show --resource-group {} --name {}".format(resource_group, job), checks=[
            JMESPathCheck('name', job),
            JMESPathCheck('properties.configuration.replicaTimeout', 200),
            JMESPathCheck('properties.configuration.replicaRetryLimit', 2),
            JMESPathCheck('properties.configuration.triggerType', "manual", case_sensitive=False),
            JMESPathCheck('properties.configuration.secrets[0].name', "testsecret"),
        ])

        # get list of Container App Jobs secrets
        job_secret_list = self.cmd("az containerapp job secret list --resource-group {} --name {}".format(resource_group, job), checks=[
            JMESPathCheck('[0].name', "testsecret"),
            ]).get_output_in_json()
        self.assertTrue(len(job_secret_list) == 1)

        # get list of Container App Jobs secrets with secret value
        job_secret_list = self.cmd("az containerapp job secret list --resource-group {} --name {} --show-values".format(resource_group, job), checks=[
            JMESPathCheck('[0].name', "testsecret"),
            JMESPathCheck('[0].value', "testsecretvalue"),
            ]).get_output_in_json()
        self.assertTrue(len(job_secret_list) == 1)

        # Show value of specified secret in a Container App Job
        self.cmd("az containerapp job secret show --resource-group {} --name {} --secret-name testsecret".format(resource_group, job), checks=[
            JMESPathCheck('name', "testsecret"),
            JMESPathCheck('value', "testsecretvalue"),
            ])

        # Update value of existing secret in a Container App Job
        self.cmd("az containerapp job secret set --resource-group {} --name {} --secret 'testsecret=testsecretvaluev2'".format(resource_group, job))

        # check for updated value of existing secret in a Container App Job
        self.cmd("az containerapp job secret show --resource-group {} --name {} --secret-name testsecret".format(resource_group, job), checks=[
            JMESPathCheck('name', "testsecret"),
            JMESPathCheck('value', "testsecretvaluev2"),
        ])

        # add secret for a Container App Job
        job_secret_list_updated = self.cmd("az containerapp job secret set --resource-group {} --name {} --secret 'testsecret2=testsecretvalue2'".format(resource_group, job)).get_output_in_json()
        self.assertTrue(len(job_secret_list_updated) == 2)

        # check for added secret in a Container App Job
        self.cmd("az containerapp job secret show --resource-group {} --name {} --secret-name testsecret2".format(resource_group, job), checks=[
            JMESPathCheck('name', "testsecret2"),
            JMESPathCheck('value', "testsecretvalue2"),
        ])

        # delete secret from a Container App Job
        self.cmd("az containerapp job secret remove --resource-group {} --name {} --secret-name testsecret2 --yes".format(resource_group, job))

        # confirm secret is deleted in a Container App Job, leaving only one secret
        job_secret_list = self.cmd("az containerapp job secret list --resource-group {} --name {} --show-values".format(resource_group, job), checks=[
            JMESPathCheck('[0].name', "testsecret"),
            JMESPathCheck('[0].value', "testsecretvaluev2"),
            ]).get_output_in_json()
        self.assertTrue(len(job_secret_list) == 1)



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
#
# Code generated by aaz-dev-tools
# --------------------------------------------------------------------------------------------

# pylint: skip-file
# flake8: noqa

from azure.cli.core.aaz import *


@register_command(
    "network watcher connection-monitor query",
)
class Query(AAZCommand):
    """Query a snapshot of the most recent connection state of a connection monitor.

    :example: List a connection monitor for the given region.
        az network watcher connection-monitor query -l westus -n MyConnectionMonitorName
    """

    _aaz_info = {
        "version": "2022-01-01",
        "resources": [
            ["mgmt-plane", "/subscriptions/{}/resourcegroups/{}/providers/microsoft.network/networkwatchers/{}/connectionmonitors/{}/query", "2022-01-01"],
        ]
    }

    AZ_SUPPORT_NO_WAIT = True

    def _handler(self, command_args):
        super()._handler(command_args)
        return self.build_lro_poller(self._execute_operations, self._output)

    _args_schema = None

    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        if cls._args_schema is not None:
            return cls._args_schema
        cls._args_schema = super()._build_arguments_schema(*args, **kwargs)

        # define Arg Group ""

        _args_schema = cls._args_schema
        _args_schema.connection_monitor_name = AAZStrArg(
            options=["-n", "--name", "--connection-monitor-name"],
            help="Connection monitor name.",
            required=True,
            id_part="child_name_1",
        )
        _args_schema.network_watcher_name = AAZStrArg(
            options=["--network-watcher-name"],
            help="The name of the Network Watcher resource.",
            required=True,
            id_part="name",
        )
        _args_schema.resource_group_name = AAZResourceGroupNameArg(
            options=["-g", "--resource-group-name"],
            help="Name of resource group. You can configure the default group using `az configure --defaults group=<name>`.",
            required=True,
        )
        return cls._args_schema

    def _execute_operations(self):
        self.pre_operations()
        yield self.ConnectionMonitorsQuery(ctx=self.ctx)()
        self.post_operations()

    @register_callback
    def pre_operations(self):
        pass

    @register_callback
    def post_operations(self):
        pass

    def _output(self, *args, **kwargs):
        result = self.deserialize_output(self.ctx.vars.instance, client_flatten=True)
        return result

    class ConnectionMonitorsQuery(AAZHttpOperation):
        CLIENT_TYPE = "MgmtClient"

        def __call__(self, *args, **kwargs):
            request = self.make_request()
            session = self.client.send_request(request=request, stream=False, **kwargs)
            if session.http_response.status_code in [202]:
                return self.client.build_lro_polling(
                    self.ctx.args.no_wait,
                    session,
                    self.on_200,
                    self.on_error,
                    lro_options={"final-state-via": "location"},
                    path_format_arguments=self.url_parameters,
                )
            if session.http_response.status_code in [200]:
                return self.client.build_lro_polling(
                    self.ctx.args.no_wait,
                    session,
                    self.on_200,
                    self.on_error,
                    lro_options={"final-state-via": "location"},
                    path_format_arguments=self.url_parameters,
                )

            return self.on_error(session.http_response)

        @property
        def url(self):
            return self.client.format_url(
                "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Network/networkWatchers/{networkWatcherName}/connectionMonitors/{connectionMonitorName}/query",
                **self.url_parameters
            )

        @property
        def method(self):
            return "POST"

        @property
        def error_format(self):
            return "ODataV4Format"

        @property
        def url_parameters(self):
            parameters = {
                **self.serialize_url_param(
                    "connectionMonitorName", self.ctx.args.connection_monitor_name,
                    required=True,
                ),
                **self.serialize_url_param(
                    "networkWatcherName", self.ctx.args.network_watcher_name,
                    required=True,
                ),
                **self.serialize_url_param(
                    "resourceGroupName", self.ctx.args.resource_group_name,
                    required=True,
                ),
                **self.serialize_url_param(
                    "subscriptionId", self.ctx.subscription_id,
                    required=True,
                ),
            }
            return parameters

        @property
        def query_parameters(self):
            parameters = {
                **self.serialize_query_param(
                    "api-version", "2022-01-01",
                    required=True,
                ),
            }
            return parameters

        @property
        def header_parameters(self):
            parameters = {
                **self.serialize_header_param(
                    "Accept", "application/json",
                ),
            }
            return parameters

        def on_200(self, session):
            data = self.deserialize_http_content(session)
            self.ctx.set_var(
                "instance",
                data,
                schema_builder=self._build_schema_on_200
            )

        _schema_on_200 = None

        @classmethod
        def _build_schema_on_200(cls):
            if cls._schema_on_200 is not None:
                return cls._schema_on_200

            cls._schema_on_200 = AAZObjectType()
            _QueryHelper._build_schema_connection_monitor_query_result_read(cls._schema_on_200)

            return cls._schema_on_200


class _QueryHelper:
    """Helper class for Query"""

    _schema_connection_monitor_query_result_read = None

    @classmethod
    def _build_schema_connection_monitor_query_result_read(cls, _schema):
        if cls._schema_connection_monitor_query_result_read is not None:
            _schema.source_status = cls._schema_connection_monitor_query_result_read.source_status
            _schema.states = cls._schema_connection_monitor_query_result_read.states
            return

        cls._schema_connection_monitor_query_result_read = _schema_connection_monitor_query_result_read = AAZObjectType()

        connection_monitor_query_result_read = _schema_connection_monitor_query_result_read
        connection_monitor_query_result_read.source_status = AAZStrType(
            serialized_name="sourceStatus",
        )
        connection_monitor_query_result_read.states = AAZListType()

        states = _schema_connection_monitor_query_result_read.states
        states.Element = AAZObjectType()

        _element = _schema_connection_monitor_query_result_read.states.Element
        _element.avg_latency_in_ms = AAZIntType(
            serialized_name="avgLatencyInMs",
        )
        _element.connection_state = AAZStrType(
            serialized_name="connectionState",
        )
        _element.end_time = AAZStrType(
            serialized_name="endTime",
        )
        _element.evaluation_state = AAZStrType(
            serialized_name="evaluationState",
        )
        _element.hops = AAZListType(
            flags={"read_only": True},
        )
        _element.max_latency_in_ms = AAZIntType(
            serialized_name="maxLatencyInMs",
        )
        _element.min_latency_in_ms = AAZIntType(
            serialized_name="minLatencyInMs",
        )
        _element.probes_failed = AAZIntType(
            serialized_name="probesFailed",
        )
        _element.probes_sent = AAZIntType(
            serialized_name="probesSent",
        )
        _element.start_time = AAZStrType(
            serialized_name="startTime",
        )

        hops = _schema_connection_monitor_query_result_read.states.Element.hops
        hops.Element = AAZObjectType()

        _element = _schema_connection_monitor_query_result_read.states.Element.hops.Element
        _element.address = AAZStrType(
            flags={"read_only": True},
        )
        _element.id = AAZStrType(
            flags={"read_only": True},
        )
        _element.issues = AAZListType(
            flags={"read_only": True},
        )
        _element.links = AAZListType(
            flags={"read_only": True},
        )
        _element.next_hop_ids = AAZListType(
            serialized_name="nextHopIds",
            flags={"read_only": True},
        )
        _element.previous_hop_ids = AAZListType(
            serialized_name="previousHopIds",
            flags={"read_only": True},
        )
        _element.previous_links = AAZListType(
            serialized_name="previousLinks",
            flags={"read_only": True},
        )
        _element.resource_id = AAZStrType(
            serialized_name="resourceId",
            flags={"read_only": True},
        )
        _element.type = AAZStrType(
            flags={"read_only": True},
        )

        issues = _schema_connection_monitor_query_result_read.states.Element.hops.Element.issues
        issues.Element = AAZObjectType()
        cls._build_schema_connectivity_issue_read(issues.Element)

        links = _schema_connection_monitor_query_result_read.states.Element.hops.Element.links
        links.Element = AAZObjectType()
        cls._build_schema_hop_link_read(links.Element)

        next_hop_ids = _schema_connection_monitor_query_result_read.states.Element.hops.Element.next_hop_ids
        next_hop_ids.Element = AAZStrType()

        previous_hop_ids = _schema_connection_monitor_query_result_read.states.Element.hops.Element.previous_hop_ids
        previous_hop_ids.Element = AAZStrType()

        previous_links = _schema_connection_monitor_query_result_read.states.Element.hops.Element.previous_links
        previous_links.Element = AAZObjectType()
        cls._build_schema_hop_link_read(previous_links.Element)

        _schema.source_status = cls._schema_connection_monitor_query_result_read.source_status
        _schema.states = cls._schema_connection_monitor_query_result_read.states

    _schema_connectivity_issue_read = None

    @classmethod
    def _build_schema_connectivity_issue_read(cls, _schema):
        if cls._schema_connectivity_issue_read is not None:
            _schema.context = cls._schema_connectivity_issue_read.context
            _schema.origin = cls._schema_connectivity_issue_read.origin
            _schema.severity = cls._schema_connectivity_issue_read.severity
            _schema.type = cls._schema_connectivity_issue_read.type
            return

        cls._schema_connectivity_issue_read = _schema_connectivity_issue_read = AAZObjectType()

        connectivity_issue_read = _schema_connectivity_issue_read
        connectivity_issue_read.context = AAZListType(
            flags={"read_only": True},
        )
        connectivity_issue_read.origin = AAZStrType(
            flags={"read_only": True},
        )
        connectivity_issue_read.severity = AAZStrType(
            flags={"read_only": True},
        )
        connectivity_issue_read.type = AAZStrType(
            flags={"read_only": True},
        )

        context = _schema_connectivity_issue_read.context
        context.Element = AAZDictType()

        _element = _schema_connectivity_issue_read.context.Element
        _element.Element = AAZStrType()

        _schema.context = cls._schema_connectivity_issue_read.context
        _schema.origin = cls._schema_connectivity_issue_read.origin
        _schema.severity = cls._schema_connectivity_issue_read.severity
        _schema.type = cls._schema_connectivity_issue_read.type

    _schema_hop_link_read = None

    @classmethod
    def _build_schema_hop_link_read(cls, _schema):
        if cls._schema_hop_link_read is not None:
            _schema.context = cls._schema_hop_link_read.context
            _schema.issues = cls._schema_hop_link_read.issues
            _schema.link_type = cls._schema_hop_link_read.link_type
            _schema.next_hop_id = cls._schema_hop_link_read.next_hop_id
            _schema.properties = cls._schema_hop_link_read.properties
            _schema.resource_id = cls._schema_hop_link_read.resource_id
            return

        cls._schema_hop_link_read = _schema_hop_link_read = AAZObjectType()

        hop_link_read = _schema_hop_link_read
        hop_link_read.context = AAZDictType(
            flags={"read_only": True},
        )
        hop_link_read.issues = AAZListType(
            flags={"read_only": True},
        )
        hop_link_read.link_type = AAZStrType(
            serialized_name="linkType",
            flags={"read_only": True},
        )
        hop_link_read.next_hop_id = AAZStrType(
            serialized_name="nextHopId",
            flags={"read_only": True},
        )
        hop_link_read.properties = AAZObjectType(
            flags={"client_flatten": True},
        )
        hop_link_read.resource_id = AAZStrType(
            serialized_name="resourceId",
            flags={"read_only": True},
        )

        context = _schema_hop_link_read.context
        context.Element = AAZStrType()

        issues = _schema_hop_link_read.issues
        issues.Element = AAZObjectType()
        cls._build_schema_connectivity_issue_read(issues.Element)

        properties = _schema_hop_link_read.properties
        properties.round_trip_time_avg = AAZIntType(
            serialized_name="roundTripTimeAvg",
            flags={"read_only": True},
        )
        properties.round_trip_time_max = AAZIntType(
            serialized_name="roundTripTimeMax",
            flags={"read_only": True},
        )
        properties.round_trip_time_min = AAZIntType(
            serialized_name="roundTripTimeMin",
            flags={"read_only": True},
        )

        _schema.context = cls._schema_hop_link_read.context
        _schema.issues = cls._schema_hop_link_read.issues
        _schema.link_type = cls._schema_hop_link_read.link_type
        _schema.next_hop_id = cls._schema_hop_link_read.next_hop_id
        _schema.properties = cls._schema_hop_link_read.properties
        _schema.resource_id = cls._schema_hop_link_read.resource_id


__all__ = ["Query"]



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

import os
from azure.cli.testsdk import CliTestError, ResourceGroupPreparer
from azure.cli.testsdk.preparers import AbstractPreparer, SingleValueReplacer
from azure.cli.testsdk.base import execute


class BatchAccountPreparer(AbstractPreparer, SingleValueReplacer):
    def __init__(self, name_prefix='clibatch', parameter_name='batch_account_name', location='eastus2',
                 resource_group_parameter_name='resource_group', skip_delete=True,
                 dev_setting_name='AZURE_CLI_TEST_DEV_BATCH_ACCT_NAME'):
        super().__init__(name_prefix, 24)
        from azure.cli.core.mock import DummyCli
        self.cli_ctx = DummyCli()
        self.parameter_name = parameter_name
        self.resource_group = None
        self.resource_group_parameter_name = resource_group_parameter_name
        self.skip_delete = skip_delete
        self.location = location
        self.dev_setting_value = os.environ.get(dev_setting_name, None)

    def create_resource(self, name, **kwargs):
        if not self.dev_setting_value:
            self.resource_group = self._get_resource_group(**kwargs)
            cmd = 'az batch account create -n {} -g {} -l {}'.format(name, self.resource_group,
                                                                     self.location)
            execute(self.cli_ctx, cmd)
            return {self.parameter_name: name}
        return {self.parameter_name: self.dev_setting_value}

    def remove_resource(self, name, **kwargs):
        if not self.dev_setting_value and not self.skip_delete:
            cmd = 'az batch account delete -n {} -g {} -y'.format(name, self.resource_group)
            execute(self.cli_ctx, cmd)

    def _get_resource_group(self, **kwargs):
        try:
            return kwargs.get(self.resource_group_parameter_name)
        except KeyError:
            template = 'To create a storage account a resource group is required. Please add ' \
                       'decorator @{} in front of this batch account preparer.'
            raise CliTestError(template.format(ResourceGroupPreparer.__name__,
                                               self.resource_group_parameter_name))


class BatchScenarioMixin:
    def set_account_info(self, account_name, group_name):
        """Returns the batch account name, key, and endpoint in a tuple."""
        key = self.get_account_key(account_name, group_name)
        endpoint = self.get_account_endpoint(account_name, group_name)
        self.kwargs.update({
            'acc_u': endpoint,
            'acc_k': key,
            'acc_n': account_name,
        })

    def get_account_key(self, *args):
        return self.cmd(
            'batch account keys list -n {} -g {} --query "secondary" -otsv'.format(*args)).output

    def get_account_endpoint(self, *args):
        endpoint = self.cmd('batch account show -n {} -g {}'.format(*args)).get_output_in_json()[
            'accountEndpoint']
        return 'https://' + endpoint

    def batch_cmd(self, cmd, **kwargs):
        cmd = cmd + ' --account-name {acc_n} --account-key "{acc_k}" --account-endpoint {acc_u}'
        return self.cmd(cmd, **kwargs)
    

class BatchMgmtScenarioMixin: 
    def set_account_info(self, account_name, group_name):
        """Returns the batch account name, key, and endpoint in a tuple."""
        endpoint = self.get_account_endpoint(account_name, group_name)
        self.kwargs.update({
            'acc_u': endpoint,
            'acc_n': account_name,
        })

  
    def get_account_endpoint(self, *args):
        endpoint = self.cmd('batch account show -n {} -g {}'.format(*args)).get_output_in_json()[
            'accountEndpoint']
        return 'https://' + endpoint

    def batch_cmd(self, cmd, **kwargs):
        cmd = cmd + ' --account-name {acc_n}  --account-endpoint {acc_u}'
        return self.cmd(cmd, **kwargs)



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
#
# Code generated by aaz-dev-tools
# --------------------------------------------------------------------------------------------

# pylint: skip-file
# flake8: noqa

from azure.cli.core.aaz import *


class ConfigureFlowLog(AAZCommand):
    """Configure flow logging on a network security group.

    :example: Enable NSG flow logs.
        az network watcher flow-log configure -g MyResourceGroup --enabled true --nsg MyNsg --storage-account MyStorageAccount

    :example: Disable NSG flow logs.
        az network watcher flow-log configure -g MyResourceGroup --enabled false --nsg MyNsg
    """

    _aaz_info = {
        "version": "2022-01-01",
        "resources": [
            ["mgmt-plane", "/subscriptions/{}/resourcegroups/{}/providers/microsoft.network/networkwatchers/{}/queryflowlogstatus", "2022-01-01"],
        ]
    }

    AZ_SUPPORT_NO_WAIT = True

    def _handler(self, command_args):
        super()._handler(command_args)
        return self.build_lro_poller(self._execute_operations, self._output)

    _args_schema = None

    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        if cls._args_schema is not None:
            return cls._args_schema
        cls._args_schema = super()._build_arguments_schema(*args, **kwargs)

        # define Arg Group ""

        _args_schema = cls._args_schema
        _args_schema.network_watcher_name = AAZStrArg(
            options=["--network-watcher-name"],
            help="The name of the network watcher resource.",
            required=True,
            id_part="name",
        )
        _args_schema.resource_group = AAZResourceGroupNameArg(
            required=True,
        )

        # define Arg Group "Parameters"

        _args_schema = cls._args_schema
        _args_schema.target_resource_id = AAZStrArg(
            options=["--target-resource-id"],
            arg_group="Parameters",
            help="The target resource where getting the flow log and traffic analytics (optional) status.",
            required=True,
        )
        return cls._args_schema

    def _execute_operations(self):
        self.pre_operations()
        yield self.NetworkWatchersGetFlowLogStatus(ctx=self.ctx)()
        self.post_operations()

    @register_callback
    def pre_operations(self):
        pass

    @register_callback
    def post_operations(self):
        pass

    def _output(self, *args, **kwargs):
        result = self.deserialize_output(self.ctx.vars.instance, client_flatten=True)
        return result

    class NetworkWatchersGetFlowLogStatus(AAZHttpOperation):
        CLIENT_TYPE = "MgmtClient"

        def __call__(self, *args, **kwargs):
            request = self.make_request()
            session = self.client.send_request(request=request, stream=False, **kwargs)
            if session.http_response.status_code in [202]:
                return self.client.build_lro_polling(
                    self.ctx.args.no_wait,
                    session,
                    self.on_200,
                    self.on_error,
                    lro_options={"final-state-via": "location"},
                    path_format_arguments=self.url_parameters,
                )
            if session.http_response.status_code in [200]:
                return self.client.build_lro_polling(
                    self.ctx.args.no_wait,
                    session,
                    self.on_200,
                    self.on_error,
                    lro_options={"final-state-via": "location"},
                    path_format_arguments=self.url_parameters,
                )

            return self.on_error(session.http_response)

        @property
        def url(self):
            return self.client.format_url(
                "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Network/networkWatchers/{networkWatcherName}/queryFlowLogStatus",
                **self.url_parameters
            )

        @property
        def method(self):
            return "POST"

        @property
        def error_format(self):
            return "ODataV4Format"

        @property
        def url_parameters(self):
            parameters = {
                **self.serialize_url_param(
                    "networkWatcherName", self.ctx.args.network_watcher_name,
                    required=True,
                ),
                **self.serialize_url_param(
                    "resourceGroupName", self.ctx.args.resource_group,
                    required=True,
                ),
                **self.serialize_url_param(
                    "subscriptionId", self.ctx.subscription_id,
                    required=True,
                ),
            }
            return parameters

        @property
        def query_parameters(self):
            parameters = {
                **self.serialize_query_param(
                    "api-version", "2022-01-01",
                    required=True,
                ),
            }
            return parameters

        @property
        def header_parameters(self):
            parameters = {
                **self.serialize_header_param(
                    "Content-Type", "application/json",
                ),
                **self.serialize_header_param(
                    "Accept", "application/json",
                ),
            }
            return parameters

        @property
        def content(self):
            _content_value, _builder = self.new_content_builder(
                self.ctx.args,
                typ=AAZObjectType,
                typ_kwargs={"flags": {"required": True, "client_flatten": True}}
            )
            _builder.set_prop("targetResourceId", AAZStrType, ".target_resource_id", typ_kwargs={"flags": {"required": True}})

            return self.serialize_content(_content_value)

        def on_200(self, session):
            data = self.deserialize_http_content(session)
            self.ctx.set_var(
                "instance",
                data,
                schema_builder=self._build_schema_on_200
            )

        _schema_on_200 = None

        @classmethod
        def _build_schema_on_200(cls):
            if cls._schema_on_200 is not None:
                return cls._schema_on_200

            cls._schema_on_200 = AAZObjectType()
            _ConfigureFlowLogHelper._build_schema_flow_log_information_read(cls._schema_on_200)

            return cls._schema_on_200


class _ConfigureFlowLogHelper:
    """Helper class for ConfigureFlowLog"""

    _schema_flow_log_information_read = None

    @classmethod
    def _build_schema_flow_log_information_read(cls, _schema):
        if cls._schema_flow_log_information_read is not None:
            _schema.flow_analytics_configuration = cls._schema_flow_log_information_read.flow_analytics_configuration
            _schema.properties = cls._schema_flow_log_information_read.properties
            _schema.target_resource_id = cls._schema_flow_log_information_read.target_resource_id
            return

        cls._schema_flow_log_information_read = _schema_flow_log_information_read = AAZObjectType()

        flow_log_information_read = _schema_flow_log_information_read
        flow_log_information_read.flow_analytics_configuration = AAZObjectType(
            serialized_name="flowAnalyticsConfiguration",
        )
        flow_log_information_read.properties = AAZObjectType(
            flags={"required": True, "client_flatten": True},
        )
        flow_log_information_read.target_resource_id = AAZStrType(
            serialized_name="targetResourceId",
            flags={"required": True},
        )

        flow_analytics_configuration = _schema_flow_log_information_read.flow_analytics_configuration
        flow_analytics_configuration.network_watcher_flow_analytics_configuration = AAZObjectType(
            serialized_name="networkWatcherFlowAnalyticsConfiguration",
        )

        network_watcher_flow_analytics_configuration = _schema_flow_log_information_read.flow_analytics_configuration.network_watcher_flow_analytics_configuration
        network_watcher_flow_analytics_configuration.enabled = AAZBoolType()
        network_watcher_flow_analytics_configuration.traffic_analytics_interval = AAZIntType(
            serialized_name="trafficAnalyticsInterval",
        )
        network_watcher_flow_analytics_configuration.workspace_id = AAZStrType(
            serialized_name="workspaceId",
        )
        network_watcher_flow_analytics_configuration.workspace_region = AAZStrType(
            serialized_name="workspaceRegion",
        )
        network_watcher_flow_analytics_configuration.workspace_resource_id = AAZStrType(
            serialized_name="workspaceResourceId",
        )

        properties = _schema_flow_log_information_read.properties
        properties.enabled = AAZBoolType(
            flags={"required": True},
        )
        properties.format = AAZObjectType()
        properties.retention_policy = AAZObjectType(
            serialized_name="retentionPolicy",
        )
        properties.storage_id = AAZStrType(
            serialized_name="storageId",
            flags={"required": True},
        )

        format = _schema_flow_log_information_read.properties.format
        format.type = AAZStrType()
        format.version = AAZIntType()

        retention_policy = _schema_flow_log_information_read.properties.retention_policy
        retention_policy.days = AAZIntType()
        retention_policy.enabled = AAZBoolType()

        _schema.flow_analytics_configuration = cls._schema_flow_log_information_read.flow_analytics_configuration
        _schema.properties = cls._schema_flow_log_information_read.properties
        _schema.target_resource_id = cls._schema_flow_log_information_read.target_resource_id


__all__ = ["ConfigureFlowLog"]



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

# pylint: disable=protected-access

from azure.cli.core.aaz import has_value
from azure.cli.command_modules.monitor.aaz.latest.monitor.autoscale._list import List as _AutoScaleList


class AutoScaleList(_AutoScaleList):

    def _output(self, *args, **kwargs):
        from azure.cli.core.aaz import AAZUndefined
        # When the name field conflicts, the name in inner layer is ignored and the outer layer is applied
        for value in self.ctx.vars.instance.value:
            if has_value(value.properties):
                value.properties.name = AAZUndefined
        result = self.deserialize_output(self.ctx.vars.instance.value, client_flatten=True)
        next_link = self.deserialize_output(self.ctx.vars.instance.next_link)
        return result, next_link
